"""Collections on controls."""
from __future__ import print_function, division

import wx
import collections
import numpy as np

from monty.collections import AttrDict

__all__ = [
    "LinspaceControl",
    "ArangeControl",
    "IntervalControl",
    "RowMultiCtrl",
    "TableMultiCtrl",
    "ListCtrlFromTable",
    "FoldPanelMgr",
]


class LinspaceControl(wx.Panel):
    """
    This control merges two `SpinCtrlDouble` and a `SpinCtrl` to allow
    the user to specify a range using the `numpy.linspace` syntax.
    """
    # Default parameters passed to SpinCtrl and SpinCtrlDouble.
    SPIN_DEFAULTS = dict(value=str(50), min=0, max=10000, initial=0)

    SPIN_DOUBLE_DEFAULTS = dict(value=str(0.0), min=0, max=10000, initial=0, inc=1)

    def __init__(self, parent, start=None, stop=None, num=1, **kwargs):
        """
        value (string)  Default value (as text).
        min (float)  Minimal value.
        max (float)  Maximal value.
        initial (float)  Initial value.
        inc (float)  Increment value.
        """
        super(LinspaceControl, self).__init__(parent, id=-1, **kwargs)

        text_opts = dict(flag=wx.ALIGN_CENTER_VERTICAL | wx.ALL, border=5)
        ctrl_opts = text_opts

        main_sizer = wx.BoxSizer(wx.HORIZONTAL)

        # start
        text = wx.StaticText(self, -1, "Start:")
        text.Wrap(-1)
        text.SetToolTipString("The starting value of the sequence.")

        p = self.SPIN_DOUBLE_DEFAULTS.copy()
        if start is not None:
            p["value"] = str(start)

        self.start_ctrl = wx.SpinCtrlDouble(self, -1, **p)

        main_sizer.Add(text, **text_opts)
        main_sizer.Add(self.start_ctrl, **ctrl_opts)

        # stop
        text = wx.StaticText(self, -1, "Stop:")
        text.Wrap(-1)
        text.SetToolTipString("The end value of the sequence")

        p = self.SPIN_DOUBLE_DEFAULTS.copy()
        if stop is not None:
            p["value"] = str(stop)

        self.stop_ctrl = wx.SpinCtrlDouble(self, -1, **p)

        main_sizer.Add(text, **text_opts)
        main_sizer.Add(self.stop_ctrl, **ctrl_opts)

        # num
        text = wx.StaticText(self, -1, "Num:")
        text.Wrap(-1)
        text.SetToolTipString("Number of samples to generate.")

        p = self.SPIN_DEFAULTS.copy()
        if num is not None:
            p["value"] = str(num)

        self.num_ctrl = wx.SpinCtrl(self, -1, **p)
        # FIXME:
        # There's a problem since the text entered in the SpinCtrl is processed
        # only when the widget looses focus. I tried the solution discussed at
        # https://groups.google.com/forum/#!topic/wxpython-users/Gud8PI6n-4E
        # but it didn't work on my Mac
        #txtctrl = self.num_ctrl.GetChildren[0]
        #txtctrl.WindowStyle |= wx.TE_PROCESS_ENTER

        main_sizer.Add(text, **text_opts)
        main_sizer.Add(self.num_ctrl, **ctrl_opts)

        self.SetSizerAndFit(main_sizer)

    def getValues(self):
        """Returns the numpy array built with numpy.linspace."""
        # FIXME Values are not updated if I edit the string in the SpinCtrl
        p = dict(start=self.start_ctrl.GetValue(),
                 stop=self.stop_ctrl.GetValue(),
                 num=self.num_ctrl.GetValue())

        return np.linspace(**p)


class ArangeControl(wx.Panel):
    """
    This control merges three `SpinCtrlDouble` controls to allow
    the user to specify a range using the `numpy.arange` syntax.
    """
    # Default parameters passed to SpinCtrlDouble.
    SPIN_DOUBLE_DEFAULTS = dict(value=str(0.0), min=0, max=10000, initial=0, inc=1)

    def __init__(self, parent, start=None, stop=None, step=None, **kwargs):
        """
        """
        super(ArangeControl, self).__init__(parent, id=-1, **kwargs)

        text_opts = dict(flag=wx.ALIGN_CENTER_VERTICAL | wx.ALL, border=5)
        ctrl_opts = text_opts

        main_sizer = wx.BoxSizer(wx.HORIZONTAL)

        # start
        text = wx.StaticText(self, -1, "Start:")
        text.Wrap(-1)
        text.SetToolTipString("Start of interval. The interval includes this value.")

        p = self.SPIN_DOUBLE_DEFAULTS.copy()
        if start is not None:
            p["value"] = str(start)

        self.start_ctrl = wx.SpinCtrlDouble(self, -1, **p)

        main_sizer.Add(text, **text_opts)
        main_sizer.Add(self.start_ctrl, **ctrl_opts)

        # stop
        text = wx.StaticText(self, -1, "Stop:")
        text.Wrap(-1)
        text.SetToolTipString("""\
End of interval. The interval does not include this value, except
in some cases where `step` is not an integer and floating point round-off affects the length of `out`.""")

        p = self.SPIN_DOUBLE_DEFAULTS.copy()
        if stop is not None:
            p["value"] = str(stop)

        self.stop_ctrl = wx.SpinCtrlDouble(self, -1, **p)

        main_sizer.Add(text, **text_opts)
        main_sizer.Add(self.stop_ctrl, **ctrl_opts)

        # step
        text = wx.StaticText(self, -1, "Step:")
        text.Wrap(-1)
        text.SetToolTipString("""\
Spacing between values. For any output `out`, this is the distance between two adjacent values, out[i+1] - out[i].""")

        p = self.SPIN_DOUBLE_DEFAULTS.copy()
        if step is not None:
            p["value"] = str(step)

        self.step_ctrl = wx.SpinCtrlDouble(self, -1, **p)

        main_sizer.Add(text, **text_opts)
        main_sizer.Add(self.step_ctrl, **ctrl_opts)

        self.SetSizerAndFit(main_sizer)

    def getValues(self):
        """Returns the numpy array built with numpy.linspace."""
        p = dict(start=self.start_ctrl.GetValue(),
                 stop=self.stop_ctrl.GetValue(),
                 step=self.step_ctrl.GetValue())

        return np.arange(**p)


class IntervalControl(wx.Panel):
    """
    This control merges two `SpinCtrlDouble` controls, a `SpinCtrl` and a combo box
    to allow the user to specify a numerical interval
    """
    # Default parameters passed to SpinCtrl and SpinCtrlDouble.
    SPIN_DEFAULTS = dict(value=str(50), min=0, max=10000, initial=0)

    SPIN_DOUBLE_DEFAULTS = dict(value=str(0.0), min=0, max=10000, initial=0, inc=0.1)

    def __init__(self, parent, start, step, num=5, choices=None, **kwargs):
        """
        Args:
            start: Initial value
            step: Step used to generate the mesh
            num: Number of points
        """
        super(IntervalControl, self).__init__(parent, id=-1, **kwargs)

        text_opts = dict(flag=wx.ALIGN_CENTER_VERTICAL | wx.ALL, border=5)
        ctrl_opts = text_opts

        main_sizer = wx.BoxSizer(wx.HORIZONTAL)

        # start
        text = wx.StaticText(self, -1, "Start:")
        text.Wrap(-1)
        text.SetToolTipString("Start of interval. The interval includes this value.")

        p = self.SPIN_DOUBLE_DEFAULTS.copy()
        p["value"] = str(start)

        self.start_ctrl = wx.SpinCtrlDouble(self, -1, **p)

        main_sizer.Add(text, **text_opts)
        main_sizer.Add(self.start_ctrl, **ctrl_opts)

        # step
        text = wx.StaticText(self, -1, "Step:")
        text.Wrap(-1)
        text.SetToolTipString("""\
Spacing between values. For any output `out`, this is the distance between two adjacent values, out[i+1] - out[i].""")

        p = self.SPIN_DOUBLE_DEFAULTS.copy()
        p["value"] = str(step)

        self.step_ctrl = wx.SpinCtrlDouble(self, -1, **p)

        main_sizer.Add(text, **text_opts)
        main_sizer.Add(self.step_ctrl, **ctrl_opts)

        # num
        text = wx.StaticText(self, -1, "Num:")
        text.Wrap(-1)
        text.SetToolTipString("Number of samples to generate.")

        p = self.SPIN_DEFAULTS.copy()
        if num is not None:
            p["value"] = str(num)

        self.num_ctrl = wx.SpinCtrl(self, -1, **p)
        # FIXME:
        # There's a problem since the text entered in the SpinCtrl is processed
        # only when the widget looses focus. I tried the solution discussed at
        # https://groups.google.com/forum/#!topic/wxpython-users/Gud8PI6n-4E
        # but it didn't work on my Mac
        #txtctrl = self.num_ctrl.GetChildren[0]
        #txtctrl.WindowStyle |= wx.TE_PROCESS_ENTER

        main_sizer.Add(text, **text_opts)
        main_sizer.Add(self.num_ctrl, **ctrl_opts)

        # Combo box to select the interval type.
        text = wx.StaticText(self, -1, "Interval type:")
        text.SetToolTipString("""\
Select the interval type:
centered if values are centered on start,
> if the lower bound of the interval is start, < if the uppper bound is start.""")

        if choices is None: choices = ["centered", ">", "<"]
        self.interval_type = wx.ComboBox(
            self, id=-1, name='Interval type type', choices=choices, value=choices[0], style=wx.CB_READONLY)

        main_sizer.Add(text, **text_opts)
        main_sizer.Add(self.interval_type, **ctrl_opts)

        self.SetSizerAndFit(main_sizer)

    def getValues(self):
        """Returns a numpy array with the interval."""
        start = self.start_ctrl.GetValue()
        step = self.step_ctrl.GetValue()
        num = self.num_ctrl.GetValue()
        int_type = self.interval_type.GetValue()

        assert num > 0
        if num == 1:
            return np.array(start)

        if int_type in [">", "<"]:
            sign = 1 if int_type == ">" else -1
            values = [start + i * sign * step for i in range(num)]
            if int_type == "<":
                values.reverse()

        elif int_type == "centered":
            v_left = [start - i * step for i in range(1, num)]
            v_left.reverse()
            v_right = [start + i * step for i in range(num)]
            values = v_left + v_right

        else:
            raise ValueError("Wrong int_type %s" % int_type)

        #print("values", values)
        return np.array(values)


class RowMultiCtrl(wx.Panel):
    """A panel with control widgets for integer, floats, ... placed on a row."""
    # Default parameters passed to SpinCtrl and SpinCtrlDouble.
    SPIN_DEFAULTS = dict(value="0", min=0, max=10000, initial=1)

    SPIN_DOUBLE_DEFAULTS = dict(value="0.0", min=-10000, max=10000, initial=0.0, inc=0.1)

    def __init__(self, parent, ctrl_params):
        """
        Args:
            ctrl_params:
                List whose items are in the form (label: params)
                where label is the name of the Spin button and
                and params is a dictionary with the arguments used
                to build the controller. Available keys are listed below.
                Note that dtype must be specified.

                ===========  ============================================================
                dtype        "f" for floats, "i" for integers, "cbox" for combo box
                tooltip      tooltip of the controller
                choices      list of possible choices (used if dtype == cbox, mandatory
                style        used if dtype == "cbox"
                ===========  ============================================================

        Example:
            RowMultiCtrl(parent, ctrl_params=[
                ("I'm an integer", dict(dtype="i", value="-1", tooltip="hello integer)),
                ("I'm a float", dict(dtype="f", value=str(1/3.))),
            ])
        """
        super(RowMultiCtrl, self).__init__(parent, -1)

        # Accepts lists or tuples as well.
        if isinstance(ctrl_params, (list, tuple)):
            ctrl_params = collections.OrderedDict(ctrl_params)

        self.ctrl_params = ctrl_params
        self.ctrls = collections.OrderedDict()

        self.main_sizer = main_sizer = wx.BoxSizer(wx.HORIZONTAL)

        for label, params in ctrl_params.items():
            params = AttrDict(**params)

            dtype = params.pop("dtype", None)
            if dtype is None:
                raise ValueError("dtype must be specified")

            txt = wx.StaticText(self, -1, label)
            txt.Wrap(-1)

            # Set the tooltip
            tooltip = params.get("tooltip", None)
            if tooltip is not None:
                txt.SetToolTipString(tooltip)

            # Create the controller and save it in self.ctrls
            if dtype == "f":
                # Initialize default values then update them with the values in params.
                opts = self.SPIN_DOUBLE_DEFAULTS.copy()
                for k in opts:
                    if k in params:
                        opts[k] = params[k]

                # Make sure value is a string and create the Ctrl
                opts["value"] = str(opts["value"])
                ctrl = wx.SpinCtrlDouble(self, id=-1, **opts)

            elif dtype == "i":
                # Initialize default values then update them with the values in params.
                opts = self.SPIN_DEFAULTS.copy()
                for k in opts:
                    if k in params:
                        opts[k] = params[k]

                # Make sure value is a string and create the Ctrl
                opts["value"] = str(opts["value"])
                ctrl = wx.SpinCtrl(self, id=-1, **opts)

            elif dtype == "cbox":
                # Combo box
                if not hasattr(params, "choices"):
                    raise ValueError("choices must be specified if dtype == cbox")
                choices = params.choices
                ctrl = wx.ComboBox(self, id=-1, choices=choices, value=choices[0],
                                   style=params.get("style", wx.CB_READONLY))
            else:
                raise ValueError("Wrong dtype %s" % str(dtype))

            self.ctrls[label] = ctrl

            hsizer = wx.BoxSizer(wx.HORIZONTAL)
            hsizer.Add(txt, 0, wx.ALIGN_CENTER_VERTICAL | wx.ALL, 5)
            hsizer.Add(ctrl, 0, wx.ALIGN_CENTER_VERTICAL | wx.ALL, 5)

            main_sizer.Add(hsizer)

        self.SetSizerAndFit(main_sizer)

    def GetParams(self):
        """Return the parameters selected by the user in a `OrderedDict`"""
        tuples = [(label, ctrl.GetValue()) for label, ctrl in self.ctrls.items()]
        return collections.OrderedDict(tuples)

    def SetParams(self, d):
        """
        Set the value of the controllers from a dictionary
        Returns the difference between the number of controllers
        and the number of keys in kwargs that have been set
        """
        for k, v in d.items():
            self.ctrls[k].SetValue(v)


class TableMultiCtrl(wx.Panel):
    """
    A panel with a table of control widgets for integer, floats, ....
    """
    def __init__(self, parent, nrows, ctrls, rowmctrl_class=None):
        """
        Args:
            nrows:
                Number of rows
            ctrls:
                List whose items are in the form (dtype: param). See RowMultiCtrl
            rowmctrl_class:
                Subclass of RowMultiCtrl used to customize the behaviour of the
                widget (optional)
        """
        super(TableMultiCtrl, self).__init__(parent, -1)

        # Allow the user to customize RowMultiCtrl
        self.rowmctrl_class = RowMultiCtrl if rowmctrl_class is None else rowmctrl_class

        self.main_sizer = wx.BoxSizer(wx.VERTICAL)

        self.ctrl_list, self.ctrls = [], ctrls
        for row in range(nrows):
            rowmc = self.rowmctrl_class(self, ctrls)
            self.main_sizer.Add(rowmc, 1, flag=wx.ALIGN_CENTER_VERTICAL | wx.ALL)
            self.ctrl_list.append(rowmc)

        self.SetSizerAndFit(self.main_sizer)

    def __getitem__(self, item):
        return self.ctrl_list.__getitem__(item)

    def __iter__(self):
        return self.ctrl_list.__iter__()

    def __len__(self):
        return len(self.ctrl_list)

    def removeRow(self, index=-1):
        """Remove row with the give index (default: last row)."""
        if index == -1: index = len(self) - 1
        if index >= len(self) or len(self) == 0: return
        #print("removing row with index: %s/%s" % (index, len(self)))

        self.main_sizer.Hide(index)
        self.main_sizer.Remove(index)
        self.ctrl_list.pop(index)
        self.main_sizer.Layout()
                                           
    def appendRow(self):
        """Add a new row."""
        rowmc = self.rowmctrl_class(self, self.ctrls)
        self.main_sizer.Add(rowmc, 1, flag=wx.ALIGN_CENTER_VERTICAL | wx.ALL)
        self.ctrl_list.append(rowmc)
        self.main_sizer.Layout()

    def GetParams(self):
        """Return the parameters selected by the user in a list of `AttrDict` dictionary"""
        olist = []
        for row in self.ctrl_list:
            olist.append(row.GetParams())

        return olist

    def SetParams(self, ilist):
        """
        Set the value of the controllers from a list of dictionaries
        """
        assert len(ilist) == len(self.ctrl_list)
        for i, d in enumerate(ilist):
            ctrl = self.ctrl_list[i]
            ctrl.SetParams(d)


import wx.lib.mixins.listctrl as listmix
from abipy.gui.awx.core import get_width_height


class ListCtrlFromTable(wx.ListCtrl, listmix.ColumnSorterMixin, listmix.ListCtrlAutoWidthMixin):
    """
    This ListCtrl displays a list of strings. Support column sorting.
    """
    def __init__(self, parent, table, **kwargs):
        """
        Args:
            parent:
                Parent window.
            table:
                Table of strings (List of lists). The first item contains the names of the columns.
            kwargs:
                enumrows:
                    If True, the first column will contain the row index (default)
        """
        enumrows = kwargs.pop("enumrows", True)
        super(ListCtrlFromTable, self).__init__(parent, id=-1, style=wx.LC_REPORT | wx.BORDER_SUNKEN, **kwargs)

        # Make sure we have a list of strings
        for i, row in enumerate(table):
            table[i] = map(str, row)

        columns = table[0]
        if enumrows:
            columns = ["#"] + columns

        for (index, col) in enumerate(columns):
            self.InsertColumn(index, col)

        # Used to store the Max width in pixels for the data in the column.
        column_widths = [get_width_height(self, s)[0] for s in columns]

        # Used by the ColumnSorterMixin, see wx/lib/mixins/listctrl.py
        self.itemDataMap = {}

        for (index, entry) in enumerate(table[1:]):
            if enumrows:
                entry = [str(index)] + entry
            self.Append(entry)
            self.itemDataMap[index] = entry
            self.SetItemData(index, index)

            w = [get_width_height(self, s)[0] for s in entry]
            column_widths = map(max, zip(w, column_widths))

        for (index, col) in enumerate(columns):
            self.SetColumnWidth(index, column_widths[index])

        # Now that the list exists we can init the other base class, see wx/lib/mixins/listctrl.py
        listmix.ColumnSorterMixin.__init__(self, len(columns))
        listmix.ListCtrlAutoWidthMixin.__init__(self)

    def GetListCtrl(self):
        """Used by the ColumnSorterMixin, see wx/lib/mixins/listctrl.py"""
        return self


import wx.lib.foldpanelbar as foldpanel

class FoldPanelMgr(foldpanel.FoldPanelBar): 
    """
    Fold panel that manages a collection of Panels

    The FoldPanelBar is a custom container class that allows multiple controls 
    to be grouped together into FoldPanelItem controls that allow them to be expanded 
    or contracted by clicking on its CaptionBar. 
    The FoldPanelBar doesn't work with layouts based on a Sizer and as such its API can get a little cumbersome, 
    because it requires you to add each control one by one and set its layout by using various flags. 
    This recipe shows how to create a custom FoldPanelBar that works with Panel objects. 
    This class will allow for you to modularize your code into Panel classes and then just add them to the FoldPanelBar 
    instead of directly adding everything to the FoldPanelBar itself.
    """ 
    def __init__(self, parent, *args, **kwargs):
        super(FoldPanelMgr, self).__init__(parent, *args, **kwargs)

    def addPanel(self, pclass, title="", collapsed=False): 
        """
        Add a panel to the manager 
        @param pclass: Class constructor (callable) 
        @keyword title: foldpanel title
        @keyword collapsed: start with it collapsed @return: pclass instance
        """ 
        fpitem = self.AddFoldPanel(title, collapsed=collapsed) 
        wnd = pclass(fpitem) 
        best = wnd.GetBestSize() 
        wnd.SetSize(best) 
        self.AddFoldPanelWindow(fpitem, wnd) 

        return wnd


if __name__ == "__main__":
    app = wx.App()
    frame = wx.Frame(None)

    #table = [["ciao", "values"], [1,2], ["3", "4"]]
    #ctrl = ListCtrlFromTable(frame, table)

    #panel = LinspaceControl(frame)
    #panel = ArangeControl(frame, start=10, stop=15, step=1)

    #panel = IntervalControl(frame, start=10, step=1, num=2)

    #panel = RowMultiCtrl(frame, [
    #    ("hello", dict(dtype="f", tooltip="Tooltip for hello", value=1/3.0)),
    #    ("integer", dict(dtype="i")),
    #    ("combo", dict(dtype="cbox", choices=["default", "another"])),
    #])

    panel = TableMultiCtrl(frame, 1, [
        ("hello", dict(dtype="f", tooltip="Tooltip for hello")),
        ("integer", dict(dtype="i", value=-1)),
    ])
    panel.appendRow()
    panel.removeRow()

    frame.Show()
    app.MainLoop()



#!/usr/bin/env python
from __future__ import print_function

import sys
import os
import time
import argparse

from subprocess import call, Popen

def str_examples():
    examples = (
      "Usage example:\n"
      "run.py               => Run all demo scripts.\n"
      "run.py -m automatic -t 7  => Run all tests, close the demo after 7 seconds.\n"
      )
    return examples


def show_examples_and_exit(err_msg=None, error_code=1):
    """Display the usage of the script."""
    sys.stderr.write(str_examples())
    if err_msg: 
        sys.stderr.write("Fatal Error\n" + err_msg + "\n")
    sys.exit(error_code)


def main():
    parser = argparse.ArgumentParser(epilog=str_examples(),formatter_class=argparse.RawDescriptionHelpFormatter)

    parser.add_argument('-m', '--mode', type=str, default="automatic",
                        help="execution mode. Either s (sequential) or a (automatic)")

    parser.add_argument('-t', '--time', type=float, default=5,
                        help="wait time seconds before running next demo.")

    options = parser.parse_args()

    # Find scripts.
    dirpath = os.path.dirname(__file__)
    scripts = []
    for fname in os.listdir(dirpath):
        if fname.endswith(".py") and fname.startswith("demo_"):
            scripts.append(os.path.join(dirpath, fname))

    python = "pythonw"
    python = "python.app"
    print("Using python:", python)

    # Run scripts depending on mode.
    if options.mode in ["s", "sequential"]:
        for script in scripts:
            retcode = call([python, script])
            if retcode != 0: break

    elif options.mode in ["a", "automatic"]:
        for script in scripts:
            p = Popen([python, script])
            time.sleep(options.time)
            p.kill()
        retcode = 0

    elif options.mode == "screenshot":
        processes = []
        for script in scripts:
            p = Popen([python, script])
            processes.append(p)
            
        time.sleep(options.time)
        for p in processes:
            p.kill()
        retcode = 0

    else:
        show_examples_and_exit(err_msg="Wrong value for mode: %s" % options.mode)

    return retcode


if __name__ == "__main__":
    sys.exit(main())



from __future__ import annotations

import panel as pn

from typing import List
from pydantic import Field, root_validator
from abipy.abio.inputs import AbinitInput
from abipy.panels.core import ply
from abipy.panels.viewers import JSONViewer
from abipy.abio.factories import ebands_input
from abipy.flowtk import TaskManager, Flow
from abipy.flowtk.flows import bandstructure_flow
from .base_models import MongoConnector, GfsFileDesc
from .flow_models import FlowModel, PresetQuery
from .gs_models import GsData, NscfData, RelaxData



class _BaseEbandsFlowModel(FlowModel):
    """
    This is the base class for models performing band structure calculations.
    It defines the three input files for the SCF/NSCF/DOS calculation
    as well as submodels storing the output results.
    """

    # These inputs can be generated either manually or from meta-parameters + factory functions.

    scf_input: AbinitInput = Field(None, description="Abinit Input file generated by AbiPy.")

    nscf_input: AbinitInput = Field(None, description="Abinit Input file generated by AbiPy.")

    dos_input: AbinitInput = Field(None, description="Abinit Input file generated by AbiPy.")

    ########
    # Output
    ########

    scf_data: GsData = Field(None, description="Results produced by the SCF run.")

    nscf_kpath_data: NscfData = Field(None, description="Results produced by the NSCF run with a k-path.")

    nscf_kmesh_data: NscfData = Field(None, description="Results produced by the NSCF run with a k-mesh.")

    with_gsr: bool = Field(False, description="Set it to True to save the GSR file in GridFS.")

    #with_den: bool = Field(False, description="Set it to True to save the DEN file in GridFS.")

    def postprocess_flow(self, flow: Flow, mng_connector: MongoConnector) -> None:
        """
        Analyze the flow and fills the model with output results.
        MongoConnector should be used only to insert files in GridFs as the final insertion is done by the caller.
        This function is called by the AbiPy Worker if the flow completed successfully.
        """
        with flow[0][0].open_gsr() as gsr:
            self.scf_data = GsData.from_gsr(gsr, mng_connector, self.with_gsr)

        with flow[0][1].open_gsr() as gsr:
            self.nscf_kpath_data = NscfData.from_gsr(gsr, mng_connector, self.with_gsr)

        if self.dos_input is not None:
            with flow[0][2].open_gsr() as gsr:
                self.nscf_kmesh_data = NscfData.from_gsr(gsr, mng_connector, self.with_gsr)

    def get_panel_view(self):
        """
        Return panel object with a view of the model.
        """
        title = self.in_structure_data.get_title()
        structure = self.in_structure_data.structure
        a, b, c = structure.lattice.abc
        alpha, beta, gamma = structure.lattice.angles
        header = f"""\
## {title}

- Lattice lengths: a = {a:.6f}, b = {b:.6f}, c = {c:.6f} Ang
- Lattice angles: α = {alpha:.3f}, β = {beta:.3f}, ɣ = {gamma:.3f} degrees
"""

        if self.is_completed:
            header += f"""\
- Pressure: {self.scf_data.pressure_gpa:.3f} GPa
- Max |Force|: {self.scf_data.max_force_ev_over_ang:.8f} eV/Ang
- Energy: {self.scf_data.energy:.4f} eV
- Energy per atom: {self.scf_data.energy_per_atom:.4f} eV
"""
            plotly_bands = ply(self.nscf_kpath_data.ebands.plotly(show=False))
        else:
            plotly_bands = pn.pane.Alert(f"Bands are not available because exec_status is `{self.flow_data.exec_status}`")

        return pn.Column(
            #self.in_structure_data.get_panel_view(),
            header,
            pn.Row(
                plotly_bands,
                pn.pane.HTML(self.scf_input._repr_html_()),
            ),
            "### MongoDB Document",
            JSONViewer(self.json(), depth=1),
            pn.layout.Divider(),
            sizing_mode="stretch_both",
        )

    @classmethod
    def get_preset_queries(cls) -> List[PresetQuery]:
        """
        Return list of dictionaries with the MongoDB queries typically used to filter documents for this model.
        Empty list if no suggestion is available.
        """
        return [
            PresetQuery.for_large_forces_or_high_pressure("scf_data", cls)
        ]

    #@classmethod
    #def mongo_aggregate_egaps(cls, collection: Collection) -> pd.DataFrame:
        #oids = cls.mng_find_completed_oids(collection)
        #    projection = [
        #        "in_structure_data
        #    }
        #
        #    fund_gap_projection = [
        #        "scf_data.fundamental_gap",
        #        "nscf_kpath_data.fundamental_gap",
        #        "nscf_kmesh_data.fundamental_gap",
        #    ]

        #    direct_gap_projection = {
        #        "scf_data.direct_gap",
        #        "nscf_kpath_data.direct_gap",
        #        "nscf_kmesh_data.direct_gap",
        #    }

        #    projection.extend(fundamental_gap_projection + direct_gap_projection)

        #    oids = cls.mng_find_completed_oids(collection)
        #    rows = []
        #    for oid in oids:
        #        doc = collection.find_one({"_id": oid}, projection)
        #        structure_data = AbipyDecoder().process_decoded(doc["in_structure_data"])
        #        #structure_data = cls.decode(doc["in_structure_data"])
        #        row = structure_data.dict4pandas()
        #        # Here I need a tool to access foo.bar instead of d["foo"]["bar"]
        #        row["fund_gap"] = min(doc[key] for key in fundamental_gap_projection)
        #        row["direct_gap"] = min(doc[key] for key in direct_gap_projection)
        #        rows.append(row)

        #    return pd.DataFrame(rows)


class EbandsFlowModelWithParams(_BaseEbandsFlowModel):
    """
    This model defines the input arguments used to build a Flow for band structure calculations
    as well as the sub models used to store the final results.
    """

    ##################
    # Input parameters
    ##################
    kppa: int = Field(1000, description="Defines the sampling used for the SCF run. Defaults to 1000 if not given.")

    ndivsm: int = Field(2, description="Number of divisions used to sample the smallest segment of the k-path.")

    spin_mode: str = Field("unpolarized", description="Spin polarization")

    charge: float = Field(0.0, description="Electronic charge added to the unit cell.")

    smearing: str = Field("fermi_dirac:0.1 eV", description="Smearing technique.")

    dos_kppa: int = Field(None,
                          description="Scalar or List of integers with the number of k-points per atom " +
                                      "to be used for the computation of the DOS (None if DOS is not wanted")

    paral_kgb: int = Field(0, description="")

    # validators
    #_normalize_name = validator('name', allow_reuse=True)(normalize)
    #_normalize_name = reuse_validator('name')(normalize)

    def build_flow(self, workdir: str, manager: TaskManager) -> Flow:
        """
        Build an AbiPy Flow using the input data available in the model and return it.

        Args:
            workdir: Working directory provided by the caller.
            manager: |TaskManager| object.

        Return: |Flow| object.
        """
        pseudos = self.pseudos_specs.get_pseudos()

        multi = ebands_input(self.in_structure_data.structure, pseudos,
                             kppa=self.kppa, nscf_nband=None, ndivsm=self.ndivsm,
                             #ecut=6, pawecutdg=None,
                             scf_nband=None, accuracy="normal",
                             spin_mode=self.spin_mode,
                             smearing=self.smearing, charge=self.charge,
                             scf_algorithm=None, dos_kppa=self.dos_kppa,
                             )

        multi.set_vars(paral_kgb=self.paral_kgb)

        if self.dos_kppa is not None:
            self.scf_input, self.nscf_input, self.dos_input = multi.split_datasets()
        else:
            self.scf_input, self.nscf_input = multi.split_datasets()

        return bandstructure_flow(workdir, self.scf_input, self.nscf_input,
                                  dos_inputs=self.dos_input, manager=manager)


class EbandsFlowModelWithInputs(_BaseEbandsFlowModel):
    """
    More flexible model for band structure calculations that requires |AbinitInput| objects
    generated by the user before building the model.
    """

    @root_validator
    def check_inputs(cls, values):
        scf_input, nscf_input = values.get("scf_input"), values.get("nscf_input")
        if scf_input is None or nscf_input is None:
            raise ValueError(f"Building a {cls.__name__} model requires both scf_input and nscf_input")
        return values

    def build_flow(self, workdir: str, manager: TaskManager) -> Flow:
        """
        Build an AbiPy Flow using the input data available in the model and return it.

        Args:
            workdir: Working directory provided by the caller.
            manager: |TaskManager| object.

        Return: |Flow| object.
        """
        return bandstructure_flow(workdir, self.scf_input, self.nscf_input,
                                  dos_inputs=self.dos_input, manager=manager)



#!/usr/bin/env python
"""
This script checks that the options in ``manager.yml``, ``scheduler.yml``,
and the environment on the local machine are properly configured.
"""

import sys
import os
import argparse
import abipy.flowtk as flowtk
import abipy.data as abidata

from monty import termcolor
from monty.termcolor import cprint
from monty.functools import prof_main
from abipy import abilab


def show_managers(options):
    """
    Print table with manager files provided by AbiPy.
    """
    from tabulate import tabulate
    table = []
    root = os.path.join(abidata.dirpath, "managers")
    yaml_paths = [os.path.join(root, f) for f in os.listdir(root) if f.endswith(".yml") and "_manager" in f]
    for path in yaml_paths:
        manager = flowtk.TaskManager.from_file(path)
        hostname = os.path.basename(path).split("_")[0]
        table.append([hostname, manager.qadapter.QTYPE, path])
        if options.verbose > 1:
            print(manager)
    print(tabulate(table, headers=["hostname", "queue-type", "filepath"], tablefmt="rst"))
    return 0


def get_epilog():
    return """\
Usage example:
    abicheck.py                ==> Test abipy installation and requirements.
    abicheck.py -m             ==> Print table with manager files provided by AbiPy.
    abicheck.py -c             ==> Install Yaml configuration files for manager and scheduler in ~/.abinit/abipy dir.
    abicheck.py --with-flow    ==> Consistency check followed by execution of AbiPy flow.
"""


def get_parser(with_epilog=False):

    parser = argparse.ArgumentParser(epilog=get_epilog() if with_epilog else "",
                                     formatter_class=argparse.RawDescriptionHelpFormatter)

    parser.add_argument('--loglevel', default="ERROR", type=str,
                        help="Set the loglevel. Possible values: CRITICAL, ERROR (default), WARNING, INFO, DEBUG")
    parser.add_argument('-V', '--version', action='version', version=abilab.__version__)
    parser.add_argument('-v', '--verbose', default=0, action='count', # -vv --> verbose=2
                        help='verbose, can be supplied multiple times to increase verbosity.')
    parser.add_argument('--no-colors', default=False, action="store_true", help='Disable ASCII colors.')
    parser.add_argument('--with-flow', default=False, action="store_true", help='Build and run small abipy flow for testing.')
    parser.add_argument("-d", '--flow-dir', type=str, default=None,
                        help='Create AbiPy flow in this directory. If None, a default directory is used,')
    parser.add_argument("-m", '--show-managers', default=False, action="store_true",
                        help="Print table with manager files provided by AbiPy.")

    parser.add_argument("-c", '--create-config', default=False, action="store_true",
                        help="Create yaml configuration files in ~/abinit/.abipy with predefined settings.")
    parser.add_argument("-f", '--force-reinstall', default=False, action="store_true",
                        help="Overwrite yaml configuration files if --create-config and files already exist.")
    return parser


@prof_main
def main():

    def show_examples_and_exit(err_msg=None, error_code=1):
        """Display the usage of the script."""
        sys.stderr.write(get_epilog())
        if err_msg:
            sys.stderr.write("Fatal Error\n" + err_msg + "\n")
        sys.exit(error_code)

    parser = get_parser(with_epilog=True)

    # Parse the command line.
    try:
        options = parser.parse_args()
    except Exception:
        show_examples_and_exit(error_code=1)

    # loglevel is bound to the string value obtained from the command line argument.
    # Convert to upper case to allow the user to specify --loglevel=DEBUG or --loglevel=debug
    import logging
    numeric_level = getattr(logging, options.loglevel.upper(), None)
    if not isinstance(numeric_level, int):
        raise ValueError('Invalid log level: %s' % options.loglevel)
    logging.basicConfig(level=numeric_level)

    if options.no_colors:
        # Disable colors
        termcolor.enable(False)

    if options.show_managers:
        # Show table with manager configuration files.
        return show_managers(options)

    if options.create_config:
        # Install Yaml configuration files for manager and scheduler.
        abilab.install_config_files(workdir=None, force_reinstall=options.force_reinstall)

    errmsg = abilab.abicheck(verbose=options.verbose)
    if errmsg:
        cprint(errmsg, "red")
        cprint("TIP: Use `--show-managers` to print the manager files provided by AbiPy.\n" +
               "If abicheck.py is failing because it cannot find the manager.yml configuration file",
               "yellow")
        return 2
    else:
        cprint("\nAbipy requirements are properly configured\n", "green")

    if not options.with_flow:
        return 0

    retcode = run_flow(options)
    if retcode == 0:
        cprint("\nTest flow completed successfully\n", "green")

    return retcode


def make_scf_nscf_inputs(paral_kgb=0):
    """Returns two input files: GS run and NSCF on a high symmetry k-mesh."""
    pseudos = abidata.pseudos("14si.pspnc")
    #pseudos = data.pseudos("Si.GGA_PBE-JTH-paw.xml")

    multi = abilab.MultiDataset(structure=abidata.cif_file("si.cif"), pseudos=pseudos, ndtset=2)

    # Global variables
    ecut = 6
    global_vars = dict(ecut=ecut,
                       nband=8,
                       timopt=-1,
                       istwfk="*1",
                       nstep=15,
                       paral_kgb=paral_kgb,
                       iomode=3,
                    )

    if multi.ispaw:
        global_vars.update(pawecutdg=2*ecut)

    multi.set_vars(global_vars)

    # Dataset 1 (GS run)
    multi[0].set_kmesh(ngkpt=[8,8,8], shiftk=[0,0,0])
    multi[0].set_vars(tolvrs=1e-6)

    # Dataset 2 (NSCF run)
    kptbounds = [
        [0.5, 0.0, 0.0], # L point
        [0.0, 0.0, 0.0], # Gamma point
        [0.0, 0.5, 0.5], # X point
    ]

    multi[1].set_kpath(ndivsm=6, kptbounds=kptbounds)
    multi[1].set_vars(tolwfr=1e-12)

    # Generate two input files for the GS and the NSCF run
    scf_input, nscf_input = multi.split_datasets()
    return scf_input, nscf_input


def run_flow(options):
    """Run test flow, return exit code."""
    import tempfile
    workdir = tempfile.mkdtemp(dir=options.flow_dir)
    cprint("Running small flow in workdir: %s" % workdir, "yellow")
    print()

    # Get the SCF and the NSCF input.
    scf_input, nscf_input = make_scf_nscf_inputs()

    # Build the flow.
    flow = flowtk.bandstructure_flow(workdir, scf_input, nscf_input, manager=None)
    flow.build_and_pickle_dump()
    scheduler = flow.make_scheduler()
    retcode = scheduler.start()
    if retcode != 0:
        cprint("Scheduler returned retcode %s" % retcode, "red")
        return retcode

    flow.show_status()
    if not flow.all_ok:
        cprint("Not all tasks in flow reached all_ok", "red")
        retcode = 1

    return retcode


if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python

import sys
#import time
#import abipy.data as abidata

from abipy.core.structure import Structure
from abipy.htc.base_models import MongoConnector, mng_insert_models
from abipy.htc.structure_models import StructureData
from abipy.htc.pseudos_models import PseudoSpecs
from abipy.htc.gs_flow_models import EbandsFlowModelWithParams
from abipy.htc.dfpt_flow_models import PhononFlowModelWithParams, PhononFlowModelWithInput
#from abipy.htc.worker import AbipyWorker

# Explicit options.
#mng_connector = MongoConnector(host="mongo.pcpm.ucl.ac.be",
#                                 db_name="abipy_GM",
#                                 collection_name="ebands",
#                                 username="gmatteo",
#                                 password="Ts5qb/UtB.22",
#                                 )

# Take options from ~/.abinit/abipy/config.yml configuration file.
mng_connector = MongoConnector.from_abipy_config(collection_name="ebands")

mng_connector = MongoConnector.for_localhost(collection_name="ebands")
print(mng_connector)

# Pseudopotential specifications.
# Note that we need to specify the table_name besides the repo name

pseudos_specs = PseudoSpecs.from_repo_table_name("ONCVPSP-PBE-SR-PDv0.4", "standard")
#pseudos_specs = PseudoSpecs.from_repo_table_name("ONCVPSP-PBE-FR-PDv0.4", "standard")


def build_collection(drop=False):
    """
    Fill a MongoDB collection with FlowModel instances defining the kind of
    calculation we want to perform and the schema used to store the output results.
    This step is mandatory before starting the AbipyWorker.
    """
    if drop:
        print(f"Building new collection {collection.full_name}")
        mng_connector.drop_collection(ask_for_confirmation=False)
    else:
        print(f"Adding documents to collection {collection.full_name}")

    # Generate list of structures.
    quick = True
    #quick = False
    if quick:
        from abipy.data.ucells import structure_from_ucell
        structures = [structure_from_ucell(name) for name in ("Si", "Si-shifted")]

    else:
        structures = [Structure.from_mpid(mpid) for mpid in (
            #"mp-149",   # Si2
            #"mp-2172",  # AlAs F-43m
            #"mp-2534",  # GaAs F-43m
            "mp-32",     # Ge-dia
            "mp-149",    # Si-dia
            "mp-2534",   # GaAs-zb
            "mp-406",    # CdTe-zb
            "mp-2691",   # CdSe-zb
            "mp-2172",   # AlAs-zb
            "mp-2176",   # ZnTe-zb
            "mp-8062",   # SiC-zb
            "mp-2469",   # CdS-zb
            "mp-1550",   # AlP-zb
            "mp-1190",   # ZnSe-zb
            "mp-2657",   # TiO$_2$-t
            "mp-5229",   # SrTiO$_3$-sc
            "mp-804",    # GaN-w
            "mp-830",    # GaN-zb
            "mp-2133",   # ZnO-w
            "mp-856",    # SnO$_2$-t
            "mp-10695",  # ZnS-zb
            "mp-1342",   # BaO-rs
            "mp-2472",   # SrO-rs
            "mp-66",     # C-dia
            "mp-661",    # AlN-w
            "mp-1639",   # BN-zb
            "mp-2605",   # CaO-rs
            "mp-1960",   # Li$_2$O
            "mp-1265",   # MgO-rs
            "mp-6947",   # SiO$_2$-t
            "mp-2542",   # BeO-w
            "mp-1138",   # LiF-rs
        )]

    # Get pseudopotential tables with hints.
    accuracy = "normal"
    #pseudos = pseudos_specs.get_pseudos()

    models = []
    for i, structure in enumerate(structures):
        in_structure_data = StructureData.from_structure(structure)

        if i == 0: EbandsFlowModelWithParams.init_collection(collection)
        model = EbandsFlowModelWithParams(in_structure_data=in_structure_data,
                                          pseudos_specs=pseudos_specs,
                                          #kppa=300,
                                          kppa=2000,
                                          with_gsr=True,
                                          with_out=True,
                                          with_log=True,
                                          )

        #if i == 0: RelaxFlowModel.init_collection(collection)
        #model = RelaxFlowModel(in_structure_data=in_structure_data, pseudos_specs=pseudos_specs,
        #                        kppa=300)

        #if i == 0: PhononFlowModelWithParams.init_collection(collection)
        #scf_input = make_scf_input(structure, pseudos, accuracy)
        #model = PhononFlowModelWithInput(in_structure_data=in_structure_data,
        #                                 pseudos_specs=pseudos_specs,
        #                                 scf_input=scf_input,
        #                                 with_becs=False,
        #                                 with_quad=False)

        #sys.exit(1)
        #print(model)
        models.append(model)

    mng_insert_models(models, collection, verbose=1)
    #mng_connector.insert_models(models, collection)


def aggregate():
    collection = mng_connector.get_collection()
    df = EbandsFlowModelWithParams.mng_aggregate_in_structures(collection)
    from abipy.abilab import print_dataframe
    print_dataframe(df)


def make_scf_input(structure, pseudos, accuracy, paral_kgb=0):
    """
    This function constructs the input file for the GS calculation:
    """
    # Crystalline AlAs: computation of the second derivative of the total energy
    from abipy import abilab
    #structure = abidata.structure_from_ucell("AlAs")
    gs_inp = abilab.AbinitInput(structure, pseudos=pseudos)

    #num_valence_electrons = gs_inp.structure.num_valence_electrons(pseudos)
    num_valence_electrons = gs_inp.num_valence_electrons
    nsppol = 2
    nband = num_valence_electrons // nsppol + 4
    nband += nband % 2

    gs_inp.set_vars(
        nband=nband,
        #nband=4,
        #ecut=2.0,
        ngkpt=[4, 4, 4],
        #nshiftk=4,
        #shiftk=[0.0, 0.0, 0.5,   # This gives the usual fcc Monkhorst-Pack grid
        #        0.0, 0.5, 0.0,
        #        0.5, 0.0, 0.0,
        #        0.5, 0.5, 0.5],
        shiftk=[0, 0, 0],
        paral_kgb=paral_kgb,
        tolvrs=1.0e-8,
        diemac=9.0,
    )

    gs_inp.set_cutoffs_for_accuracy(accuracy)
    #gs_inp.set_auto_scf_nband(nsppol=1, nspinor=1, nspden=1, occopt, tsmear)
    return gs_inp


if __name__ == "__main__":
    try:
        command = sys.argv[1]
    except IndexError:
        print("Available commands: `build`, `add`")
        sys.exit(1)

    if command == "build":
        build_collection(drop=True)
    elif command == "add":
        build_collection(drop=False)
    elif command == "aggregate":
        aggregate()
    else:
        raise ValueError(f"Invalid command: {command}")



"""Collection of widgets that allow the user to interact with list of K-points."""

import wx
import wx.lib.newevent

import abipy.gui.awx as awx
import wx.lib.mixins.listctrl as listmix


class KpointsListCtrl(wx.ListCtrl, listmix.ColumnSorterMixin):
    """
    ListCtrl that allows the user to interact with a list of k-points.
    Support column sorting.
    """
    def __init__(self, parent, kpoints, **kwargs):
        """
        Args:
            parent:
                Parent window.
            kpoints:
                List of `Kpoint` instances.
        """
        super(KpointsListCtrl, self).__init__(parent, id=-1, style=wx.LC_REPORT | wx.BORDER_SUNKEN, **kwargs)

        self.kpoints = kpoints

        columns = ["#", 'Reduced coordinates', 'Weight', 'Name']

        for (index, col) in enumerate(columns):
            self.InsertColumn(index, col)

        # Used to store the Max width in pixels for the data in the column.
        column_widths = [awx.get_width_height(self, s)[0] for s in columns]

        # Used by the ColumnSorterMixin, see wx/lib/mixins/listctrl.py
        self.itemDataMap = {}

        for (index, kpt) in enumerate(self.kpoints):
            entry = ["%d\t\t" % index,
                     "[%.5f,  %.5f,  %.5f]\t\t" % tuple(c for c in kpt.frac_coords),
                     "%.3f\t\t" % kpt.weight,
                     "%s" % kpt.name,
                     ]
            self.Append(entry)
            self.itemDataMap[index] = entry
            self.SetItemData(index, index)

            w = [awx.get_width_height(self, s)[0] for s in entry]
            column_widths = map(max, zip(w, column_widths))

        for (index, col) in enumerate(columns):
            self.SetColumnWidth(index, column_widths[index])

        # Now that the list exists we can init the other base class, see wx/lib/mixins/listctrl.py
        listmix.ColumnSorterMixin.__init__(self, len(columns))

    def GetListCtrl(self):
        """Used by the ColumnSorterMixin, see wx/lib/mixins/listctrl.py"""
        return self

    def getSelectedKpoint(self):
        """
        Returns the kpoint selected by the user.
        None if no selection has been done.
        """
        # Get selected index, map to index in kpoints and return the kpoint.
        item = self.GetFirstSelected()
        if item == -1: return None
        index = self.GetItemData(item)
        return self.kpoints[index]


class KpointsPanel(awx.Panel):
    """
    A panel with a list of k-points and a structure.
    Provides popup menus for retrieving information on a single k-point.
    """
    def __init__(self, parent, structure, kpoints, **kwargs):
        """
        Args:
            parent:
                Parent window.
            structure:
                `Structure` object.
            kpoints:
                `KpointList` object.
        """
        super(KpointsPanel, self).__init__(parent, **kwargs)

        self.klist_ctrl = KpointsListCtrl(self, kpoints)
        self.structure = structure

        self.klist_ctrl.Bind(wx.EVT_LIST_ITEM_RIGHT_CLICK, self.onRightClick)

        main_sizer = wx.BoxSizer(wx.VERTICAL)
        main_sizer.Add(self.klist_ctrl, 1, wx.ALL | wx.EXPAND | wx.ALIGN_CENTER_VERTICAL, 5)
        self.SetSizerAndFit(main_sizer)

    def onRightClick(self, event):
        """Generate the popup menu."""
        popup_menu = self.makePopupMenu()
        self.PopupMenu(popup_menu, event.GetPoint())
        popup_menu.Destroy()

    def getSelectedKpoint(self):
        return self.klist_ctrl.getSelectedKpoint()

    def makePopupMenu(self):
        """
        Build and return a popup menu. Subclasses can extend or replace this base method.
        """
        self.ID_POPUP_LITTLEGROUP = wx.NewId()
        self.ID_POPUP_STAR = wx.NewId()

        menu = wx.Menu()
        menu.Append(self.ID_POPUP_LITTLEGROUP, "Little group")
        menu.Append(self.ID_POPUP_STAR, "Show star")

        # Associate menu/toolbar items with their handlers.
        menu_handlers = [
            (self.ID_POPUP_LITTLEGROUP, self.onLittleGroup),
            (self.ID_POPUP_STAR, self.onShowStar),
        ]

        for combo in menu_handlers:
            mid, handler = combo[:2]
            self.Bind(wx.EVT_MENU, handler, id=mid)

        return menu

    def onLittleGroup(self, event):
        """Show a table with the character table of the little group."""
        kpoint = self.klist_ctrl.getSelectedKpoint()
        if kpoint is None: return
        ltk = self.structure.abi_spacegroup.find_little_group(kpoint)
        table, header = ltk.bilbao_character_table()
        awx.SimpleGridFrame(self, table, title=header, labels_from_table=True).Show()

    def onShowStar(self, event):
        """Show a new `KpointsFrame` with the list of points in the star of the selected k-point."""
        kpoint = self.klist_ctrl.getSelectedKpoint()
        if kpoint is None: return
        star = kpoint.compute_star(self.structure.abi_spacegroup.fm_symmops)
        KpointsFrame(self, self.structure, star, title="Star of point: " + str(star.base_point)).Show()


class KpointsFrame(awx.Frame):
    """A frame with a `KpointsPanel`."""
    def __init__(self, parent, structure, kpoints, **kwargs):
        """
        Args:
            parent:
                Parent window.
            structure:
                `Structure` object.
            kpoints:
                `KpointList` object.
        """
        super(KpointsFrame, self).__init__(parent, **kwargs)

        self.panel = KpointsPanel(self, structure, kpoints)


class SpinKpointBandPanel(awx.Panel):
    """
    This panel shows information on the k-points and the set of bands, spins.
    Useful if we want to allow the user to select the set of indices (spin, kpt_idx, band).
    """
    # Command event used to signal that the spin-kpoint-band has been selected.
    SkbActivatedEvent, MYEVT_SKB_ACTIVATED = wx.lib.newevent.NewCommandEvent()

    def __init__(self, parent, structure, nsppol, kpoints, mband, bstart=0, **kwargs):
        """
        Args:
            structure:
                Structure object.
            nsppol:
                Number of spins.
            kpoints:
                List of `Kpoint` instances.
            mband:
                Maximum band index.
            bstart:
                First band index.
        """
        super(SpinKpointBandPanel, self).__init__(parent, style=wx.LC_REPORT | wx.BORDER_SUNKEN, **kwargs)

        self.parent = parent
        self.nsppol, self.mband = nsppol, mband
        self.bstart = bstart

        # Build UI
        main_sizer = wx.BoxSizer(wx.VERTICAL)

        hsizer1 = wx.BoxSizer(wx.HORIZONTAL)

        band_label = wx.StaticText(self, -1, "Band:", wx.DefaultPosition, wx.DefaultSize, 0)
        band_label.Wrap(-1)
        hsizer1.Add(band_label, 0, wx.ALIGN_CENTER_VERTICAL | wx.TOP | wx.BOTTOM | wx.LEFT, 5)

        self.band_cbox = wx.ComboBox(self, -1, choices=map(str, range(self.bstart, self.mband)))
        hsizer1.Add(self.band_cbox, 0, wx.ALL, 5)

        spin_label = wx.StaticText(self, -1, "Spin:", wx.DefaultPosition, wx.DefaultSize, 0)
        spin_label.Wrap(-1)
        hsizer1.Add(spin_label, 0, wx.ALIGN_CENTER_VERTICAL | wx.TOP | wx.BOTTOM | wx.LEFT, 5)

        self.spin_cbox = wx.ComboBox(self, -1, choices=map(str, range(self.nsppol)))
        hsizer1.Add(self.spin_cbox, 0, wx.ALL, 5)

        main_sizer.Add(hsizer1, 0, wx.ALIGN_CENTER_HORIZONTAL, 5)

        hsizer2 = wx.BoxSizer(wx.HORIZONTAL)

        kpoint_label = wx.StaticText(self, -1, "Kpoint:", wx.DefaultPosition, wx.DefaultSize, 0)
        kpoint_label.Wrap(-1)
        hsizer2.Add(kpoint_label, 0, wx.ALIGN_CENTER_VERTICAL | wx.ALL, 5)

        self.kpanel = KpointsPanel(self, structure, kpoints)
        klist_ctrl = self.kpanel.klist_ctrl

        hsizer2.Add(self.kpanel, 1, wx.ALL | wx.EXPAND | wx.ALIGN_CENTER_VERTICAL, 5)
        main_sizer.Add(hsizer2, 1, wx.EXPAND | wx.ALIGN_CENTER_HORIZONTAL, 5)

        self.SetSizerAndFit(main_sizer)

        # Connect the events whose callback will be set by the client code.
        klist_ctrl.Bind(wx.EVT_LIST_ITEM_ACTIVATED, self.OnItemActivated)
        #klist_ctrl.Bind(wx.EVT_LIST_ITEM_RIGHT_CLICK, self.OnRightClick)
        #klist_ctrl.Bind(wx.EVT_LIST_ITEM_SELECTED, self.OnItemSelected)

    def getSelectedKpoint(self):
        return self.kpanel.getSelectedKpoint()

    def GetSKB(self):
        """Returns the tuple (spin, kpoint, band) selected by the user."""
        spin = int(self.spin_cbox.GetValue())
        kpoint = self.getSelectedKpoint()
        band = int(self.band_cbox.GetValue())

        # Default values if no item is selected.:
        if spin == wx.NOT_FOUND: spin = 0
        if band == wx.NOT_FOUND: band = 0

        return spin, kpoint, band

    def OnItemActivated(self, event):
        """Fire SkbActivatedEvent."""
        #print("Firing SkbActivatedEvent")
        event = self.SkbActivatedEvent(id=-1, skb=self.GetSKB())
        wx.PostEvent(self.parent, event)



import os
import collections
import wx
import netCDF4
import numpy as np
import abipy.gui.awx as awx
import wx.lib.mixins.listctrl as listmix
import wx.lib.agw.flatnotebook as fnb
import wx.lib.dialogs as wxdg
import wx.lib.newevent

from monty.string import list_strings


#def ndcontract_var(var):
#    """
#    Remove fake dimensions from a Netcdf Variable.
#
#    Returns:
#        data:
#            ndarray
#        dimensions:
#            list with the name of the dimensions in data.
#    """
#    shape, dimensions = [], []
#    for num, name in zip(var.shape, var.dimensions):
#        if num > 1:
#            shape.append(num)
#            dimensions.append(name)
#
#    # Reshape array.
#    data = np.reshape(var[:], shape)
#
#    return data, dimensions


class AttrDict(dict):
    """
    Allows to access dict keys as obj.foo in addition to the traditional way
    obj['foo']"
    """
    def __init__(self, *args, **kwargs):
        super(AttrDict, self).__init__(*args, **kwargs)
        self.__dict__ = self

    def copy(self):
        newd = super(AttrDict, self).copy()
        return self.__class__(**newd)


class SimpleTextViewer(awx.Frame):
    """Very simple frame that displays text (string) in read-only mode."""
    def __init__(self, parent, text, **kwargs):
        super(SimpleTextViewer, self).__init__(parent, **kwargs)
        wx.TextCtrl(self, -1, text, style=wx.TE_MULTILINE|wx.TE_LEFT|wx.TE_READONLY)


# Command event used to signal that the value of the selected variable/dimension
# should be plotted as function of the file index.
CompareEvent, EVT_COMPARE = wx.lib.newevent.NewCommandEvent()

def getSelected(wxlist):
    """Gets the selected items from a list object."""
    selected = []

    item = -1
    while True:
        item = wxlist.GetNextItem(item, wx.LIST_NEXT_ALL, wx.LIST_STATE_SELECTED)
        if item == -1: break
        selected.append(item)

    return selected


def getColumnText(wxlist, index, col):
    """Gets the text from the specified column entry in a list."""
    item = wxlist.GetItem(index, col)
    return item.GetText()


class NcViewerFrame(wx.Frame):
    """
    This frame allows the user to inspect the dimensions
    and the variables reported in  a netcdf file.
    """
    def __init__(self, parent, filepaths=(), **kwargs):
        """
        Args:
            parent:
                parent window.
            filepaths:
                String or list of strings with the path of the nc files to open
                Empty tuple if no file should be opened during the initialization of the frame.
        """
        if "size" not in kwargs:
            kwargs["size"] = (1200, 800)

        super(NcViewerFrame, self).__init__(parent, id=-1, **kwargs)

        # This combination of options for config seems to work on my Mac.
        self.config = wx.FileConfig(appName=self.codename, localFilename=self.codename + ".ini",
                                    style=wx.CONFIG_USE_LOCAL_FILE)

        # Build menu, toolbar and status bar.
        self.makeMenu()
        self.makeToolBar()
        self.statusbar = self.CreateStatusBar()

        # Open netcdf files.
        filepaths, datasets = list_strings(filepaths), []
        filepaths = map(os.path.abspath, filepaths)

        for path in filepaths:
            datasets.append(netCDF4.Dataset(path, mode="r"))
            self.AddFileToHistory(path)

        # Create the notebook (each file will have its own tab).
        panel = wx.Panel(self, -1)
        try:
            self.notebook = fnb.FlatNotebook(panel, -1, style=fnb.FNB_NAV_BUTTONS_WHEN_NEEDED)
        except AttributeError:
            self.notebook = fnb.FlatNotebook(panel, -1)

        for path, dataset in zip(filepaths, datasets):
            tab = NcFileTab(self.notebook, dataset)
            self.notebook.AddPage(tab, os.path.basename(path))

        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(self.notebook, 1, wx.EXPAND, 5)
        panel.SetSizerAndFit(sizer)

        self.Bind(wx.EVT_CLOSE, self.OnExit)

        # Intercept the command event associated to variable/dimension comparison.
        self.Bind(EVT_COMPARE, self.OnCompare)

    @property
    def codename(self):
        """Name of the application."""
        return "wxncview"

    @property
    def num_tabs(self):
        """Number of tabs in the notebook."""
        return self.notebook.GetPageCount()

    @property
    def datasets(self):
        """List of Netcdf4 `Dataset`"""
        datasets = []
        for page in range(self.notebook.GetPageCount()):
            tab = self.notebook.GetPage(page)
            datasets.append(tab.dataset)
        return datasets

    def makeMenu(self):
        """Creates the main menu."""
        self.menu_bar = wx.MenuBar()

        self.menu_bar.Append(self.createFileMenu(), "&File")
        self.menu_bar.Append(self.createPlotMenu(), "&Plot")
        self.menu_bar.Append(self.createHelpMenu(), "&Help")

        self.SetMenuBar(self.menu_bar)

    def createFileMenu(self):
        """Creates the file menu."""
        file_menu = wx.Menu()

        # File Menu ID's
        self.idOPEN = wx.NewId()
        self.idCLOSE = wx.NewId()
        self.idEXIT = wx.NewId()

        file_menu.Append(self.idOPEN, "&Open", "Open netcdf file")
        file_menu.Append(self.idCLOSE, "&Close", "Close netcdf file")

        file_history = self.file_history = wx.FileHistory(8)
        file_history.Load(self.config)
        recent = wx.Menu()
        file_history.UseMenu(recent)
        file_history.AddFilesToMenu()
        file_menu.AppendMenu(-1, "&Recent Files", recent)
        self.Bind(wx.EVT_MENU_RANGE, self.OnFileHistory, id=wx.ID_FILE1, id2=wx.ID_FILE9)

        file_menu.AppendSeparator()
        file_menu.Append(self.idEXIT, "E&xit", "Exit wxnciew")

        self.Bind(wx.EVT_MENU, self.OnOpen, id=self.idOPEN)
        self.Bind(wx.EVT_MENU, self.OnClose, id=self.idCLOSE)
        self.Bind(wx.EVT_MENU, self.OnExit, id=self.idEXIT)
        return file_menu

    def createPlotMenu(self):
        """Creates the plot menu."""
        # Plot Menu ID's
        plot_menu = wx.Menu()

        # Options for 2D-plots
        self.twod_menu = twod_menu = wx.Menu()
        for plot_mode in ["line", "scatter"]:
            _id =  wx.NewId()
            twod_menu.AppendRadioItem(_id, plot_mode)
        plot_menu.AppendMenu(-1, '2-D plots', twod_menu)

        # Options for image plots
        self.image_menu = image_menu = wx.Menu()
        for image_mode in ["intensity", "contour", "rgb"]:
            _id =  wx.NewId()
            image_menu.AppendRadioItem(_id, image_mode)
        plot_menu.AppendMenu(-1, 'Image mode', image_menu)

        # Options for complex data.
        self.cplx_menu = cplx_menu = wx.Menu()
        for cplx_mode in ["None", "Abs", "Real", "Imag"]:
            _id =  wx.NewId()
            cplx_menu.AppendRadioItem(_id, cplx_mode)
        plot_menu.AppendMenu(-1, 'Complex', cplx_menu)

        return plot_menu

    def createHelpMenu(self):
        """Create the help menu."""
        # Help Menu ID's
        self.idABOUT = wx.NewId()

        help_menu = wx.Menu()
        help_menu.Append(self.idABOUT, "&About", "About wxncview")
        self.Bind(wx.EVT_MENU, self.OnAbout, id=self.idABOUT)

        return help_menu

    def makeToolBar(self):
        """Creates the toolbar."""
        self.toolbar = self.CreateToolBar()
        self.toolbar.SetToolBitmapSize(wx.Size(32, 32))

        #bitmap = icons.connect.getBitmap()
        #self.toolbar.AddSimpleTool(self.idTOOL_CONNECT, bitmap, shortHelpString=_("Normal Connect"))
        #bitmap = icons.quick_connect.getBitmap()

        #self.dir_entry_box = wx.ComboBox(self.toolbar, self.idTOOL_ENTRYBOX,
        #    size=wx.Size(350, -1), style=wx.CB_DROPDOWN)
        #self.binary_button = wx.RadioButton(self.toolbar, self.idTOOL_BINARY, _("BINARY"))
        #self.ascii_button = wx.RadioButton(self.toolbar, self.idTOOL_ASCII, _("ASCII"))
        #config = utils.getApplicationConfiguration()
        #if config['transfer_type'] == protocol.ProtocolInterface.BINARY:
        #    self.binary_button.SetValue(True)
        #else:
        #    self.ascii_button.SetValue(True)

        #utils.addHorizontalSpaceTool(self.toolbar, 75)
        #self.toolbar.AddControl(self.dir_entry_box)
        #utils.addHorizontalSpaceTool(self.toolbar, 5)
        #self.toolbar.AddControl(self.binary_button)
        #utils.addHorizontalSpaceTool(self.toolbar, 5)
        #self.toolbar.AddControl(self.ascii_button)

        #self.Bind(wx.EVT_TOOL, self.onConnect, id=self.idTOOL_CONNECT)
        #self.Bind(wx.EVT_COMBOBOX, self.onComboSelect, id=self.idTOOL_ENTRYBOX)
        self.toolbar.Realize()

    def AddFileToHistory(self, filepath):
        """Add the absolute filepath to the file history."""
        self.file_history.AddFileToHistory(filepath)
        self.file_history.Save(self.config)
        self.config.Flush()

    def GetPlotOptions(self):
        """Returns a dictionary with the options specified in the PlotMenu"""
        d = AttrDict()

        for radio in self.twod_menu.GetMenuItems():
            if radio.IsChecked(): d["plot_mode"] = radio.GetItemLabel()

        for radio in self.image_menu.GetMenuItems():
            if radio.IsChecked(): d["image_mode"] = radio.GetItemLabel()

        for radio in self.cplx_menu.GetMenuItems():
            if radio.IsChecked(): d["cplx_mode"] = radio.GetItemLabel()

        return d

    def OnAbout(self, event):
        """Displays the about window box."""
        # TODO
        #about = aboutwin.AboutWindow(self)
        #ret = about.ShowModal()

        #awx.makeAboutBox(codename=self.codename, version=self.VERSION,
        #                 description="", developers="M. Giantomassi")

    def read_file(self, filepath):
        """Open a netcdf file, create new tab and save the file in the history."""
        try:
            notebook = self.notebook
            dataset = netCDF4.Dataset(filepath, mode="r")
            tab = NcFileTab(notebook, dataset)
            notebook.AddPage(tab, os.path.basename(filepath))
            # don't know why but this does not work!
            notebook.Refresh()
            notebook.SetSelection(notebook.GetPageCount())
            self.AddFileToHistory(filepath)
        except:
            raise

    def OnOpen(self, event):
        """Open FileDialog to allow the user to select a netcdf file."""
        dialog = wx.FileDialog(self, wildcard="*.nc")
        if dialog.ShowModal() == wx.ID_CANCEL: return

        filepath = os.path.abspath(dialog.GetPath())
        self.read_file(filepath)

    def OnClose(self, event):
        """
        Remove the active tab from the notebook and
        close the corresponding netcdf file,
        """
        notebook = self.notebook
        if notebook.GetPageCount() == 0: return
        idx = notebook.GetSelection()
        if idx == -1: return None

        # Close the file
        tab = notebook.GetPage(idx)
        tab.dataset.close()

        # Remove tab.
        notebook.DeletePage(idx)
        notebook.Refresh()

    def OnExit(self, event):
        """Close the netcdf files and exit the application."""
        try:
            for index in range(self.notebook.GetPageCount()):
                tab = self.notebook.GetPage(index)
                try:
                    tab.dataset.close()
                except:
                    pass
        finally:
            self.Destroy()

    def OnFileHistory(self, event):
        """Open one of the file listed in the file_history."""
        fileNum = event.GetId() - wx.ID_FILE1
        filepath = self.file_history.GetHistoryFile(fileNum)
        self.read_file(filepath)

    def OnCompare(self, event):
        """
        Callback triggered by the `compare` option in the popup menu of the panels
        Plote the netcdf variable/dimension as function of the file index.
        """
        # Get the name of the variable.
        name, obj = event.name, event.obj

        # Define the function to use the get the value of the dimension/variable.
        if isinstance(obj, netCDF4.Variable):
            def extract(dataset, name):
                return dataset.variables[name][:]

        elif isinstance(obj, netCDF4.Dimension):
            def extract(dataset, name):
                return np.array(len(dataset.dimensions[name]))

        else:
            raise ValueError("Don't know how to handle %s" % repr(obj))

        # Extract data. Make sure all the shapes are equal.
        data, shape = [], None
        for dataset in self.datasets:
            v = extract(dataset, name)
            data.append(v)
            if shape is None:
                shape = v.shape
            else:
                assert shape == v.shape

        # Plot data. Two Branches for scalars and arrays.
        opts = self.GetPlotOptions()

        is_scalar = not shape or (len(shape) == 1 and shape[0] == 1)
        #print(shape)

        from wxmplot import PlotFrame
        if is_scalar:
            frame = PlotFrame(parent=self)
            xx = range(len(self.datasets))

            if opts.plot_mode == "line":
                frame.plot(xx, data)
            else:
                frame.scatterplot(xx, data)

            frame.set_xlabel("File index")
            frame.set_ylabel(name)

            frame.Show()

        else:
            # Open new frame to allow the user to specify how to handle the array.
            ArrayComparisonFrame(self, name, data).Show()


class NcFileTab(wx.Panel):
    """Tab showing information on the netcdf file."""
    def __init__(self, parent, dataset, **kwargs):
        """
        Args:
            parent:
                parent window.
            dataset:
                Netcdf4 `Dataset`.
        """
        super(NcFileTab, self).__init__(parent, -1, **kwargs)
        self.dataset = dataset

        splitter = wx.SplitterWindow(self, -1, style=wx.SP_3DSASH)

        self.dims_panel = DimsPanel(splitter, dataset)
        self.vars_panel = VarsPanel(splitter, dataset)
        splitter.SplitVertically(self.dims_panel, self.vars_panel)
        splitter.SetSashGravity(0.1)

        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(splitter, 1, wx.EXPAND, 5)
        self.SetSizerAndFit(sizer)


class BasePanel(wx.Panel, listmix.ColumnSorterMixin):
    """Mixin class providing helper functions."""

    @property
    def viewer_frame(self):
        try:
            return self._viewer_frame
        except AttributeError:
            self._viewer_frame = self.getParentWithType(NcViewerFrame)
            return self._viewer_frame

    def makePopupMenu(self):
        """
        Creates a popup menu containing the list of supported actions for local file objects.
        """
        # Popup IDs
        self.idPOPUP_GETINFO = wx.NewId()
        self.idPOPUP_COMPARE = wx.NewId()

        menu = wx.Menu()
        menu.Append(self.idPOPUP_GETINFO, "Get Info")
        self.Bind(wx.EVT_MENU, self.OnGetInfo, id=self.idPOPUP_GETINFO)

        if self.viewer_frame.num_tabs > 1:
            menu.Append(self.idPOPUP_COMPARE, "Compare")
            self.Bind(wx.EVT_MENU, self.OnCompare, id=self.idPOPUP_COMPARE)

        return menu

    def OnRightClick(self, event):
        """Creates a popup menu at the location of the right click."""
        menu = self.makePopupMenu()
        self.PopupMenu(menu, event.GetPosition())

    def OnColClick(self, event):
        event.Skip()

    def GetListCtrl(self):
        """Used by the ColumnSorterMixin, see wx/lib/mixins/listctrl.py"""
        return self.list

    def GetPlotOptions(self):
        """
        Dictionary with the options for the plots taken from
        the MenuBar of the parent `NcViewerFrame`
        """
        frame = self.ncview_frame
        return frame.GetPlotOptions()

    @property
    def ncview_frame(self):
        """The parent frame `NcViewFrame`."""
        try:
            return self._ncview_frame

        except AttributeError:
            parent = self.GetParent()
            while True:
                if parent is None:
                    raise RuntimeError("Cannot find NcViewerFrame, got None parent!")

                if isinstance(parent, NcViewerFrame):
                    break
                else:
                    parent = parent.GetParent()

            self._ncview_frame = parent
            return self._ncview_frame

    def getParentWithType(self, cls):
        """
        Returns the first parent window of type cls.

        Raises:
            RuntimeError if we have reached the head of the linked list.
        """
        parent = self.GetParent()

        while True:
            if parent is None:
                raise RuntimeError("Cannot find parent with class %s, reached None parent!" % cls)

            if isinstance(parent, cls):
                return parent
            else:
                parent = parent.GetParent()


class DimsPanel(BasePanel):
    """Panel with the netcdf dimensions."""
    def __init__(self, parent, dataset, **kwargs):
        """
        Args:
            parent:
                Parent window.
            dataset:
                Netcdf4 `Dataset`.
        """
        super(DimsPanel, self).__init__(parent, -1, **kwargs)
        self.dataset = dataset

        text = wx.StaticText(self, -1, "Number of dimensions: %d" % len(dataset.dimensions))

        columns = ["name", "value"]
        self.list = wx.ListCtrl(self, -1, style=wx.LC_REPORT | wx.SUNKEN_BORDER)

        for col, name in enumerate(columns):
            self.list.InsertColumn(col, name)

        # Used to store the Max width in pixels for the data in the column.
        column_widths = [awx.get_width_height(self, s)[0] for s in columns]

        self.id2entry = {}
        for idx, (name, dim) in enumerate(dataset.dimensions.items()):
            entry = map(str, [name, len(dim)])
            self.list.Append(entry)
            self.list.SetItemData(idx, idx)
            self.id2entry[idx] = entry

            w = [awx.get_width_height(self, s)[0] for s in entry]
            column_widths = map(max, zip(w, column_widths))

        for index, col in enumerate(columns):
            self.list.SetColumnWidth(index, column_widths[index])

        # Now that the list exists we can init the other base class, see wx/lib/mixins/listctrl.py
        self.itemDataMap = self.id2entry
        listmix.ColumnSorterMixin.__init__(self, len(columns))
        self.Bind(wx.EVT_LIST_COL_CLICK, self.OnColClick, self.list)

        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(text, 0, wx.EXPAND, 5)
        sizer.Add(self.list, 1, wx.EXPAND, 5)
        self.SetSizerAndFit(sizer)

        # Connect the events.
        self.list.Bind(wx.EVT_LIST_ITEM_RIGHT_CLICK, self.OnRightClick)
        self.list.Bind(wx.EVT_LIST_ITEM_ACTIVATED, self.OnItemActivated)

    def OnItemActivated(self, event):
        """NoP"""

    def OnCompare(self, event):
        """Trigger CompareEvent."""
        item = getSelected(self.list)[0]
        name = getColumnText(self.list, item, 0)
        dim = self.dataset.dimensions[name]

        # Create and post the event to trigger the refresh of the GUI
        event = CompareEvent(id=-1, name=name, obj=dim)
        wx.PostEvent(self.viewer_frame, event)

    def OnGetInfo(self, event):
        """Show extra information on the selected variables."""
        selected = getSelected(self.list)

        lines = []
        for item in selected:
            var_name = getColumnText(self.list, item, 0)
            var = self.dataset.dimensions[var_name]
            lines.append(str(var))
            #lines.append(ncvar_info(var))

        wxdg.ScrolledMessageDialog(self, "\n".join(lines), caption="Variable Metadata", style=wx.MAXIMIZE_BOX).Show()


class VarsPanel(BasePanel):
    """
    Panel with the netcdf variables.
    Provides popup menu to inspect/plot the selected variable.
    """
    def __init__(self, parent, dataset, **kwargs):
        """
        Args:
            parent:
                Parent window.
            dataset:
                Netcdf4 `Dataset`.
        """
        super(VarsPanel, self).__init__(parent, -1, **kwargs)

        self.dataset = dataset

        text = wx.StaticText(self, -1, "Number of variables: %d" % len(dataset.variables))

        columns = ["name", "dtype", "shape", "value", "dimensions"]
        self.list = wx.ListCtrl(self, -1, style=wx.LC_REPORT | wx.SUNKEN_BORDER)

        for col, name in enumerate(columns):
            self.list.InsertColumn(col, name)

        # Used to store the Max width in pixels for the data in the column.
        column_widths = [awx.get_width_height(self, s)[0] for s in columns]

        self.id2entry = {}
        for idx, (name, var) in enumerate(dataset.variables.items()):
            # Show the value of scalars.
            if not var.shape:
                value = str(var[:]).replace("[", "").replace("]", "").strip()
            else:
                value = "[...]"

            entry = map(str, [name, var.dtype, var.shape, value, var.dimensions])
            self.list.Append(entry)
            self.list.SetItemData(idx, idx)
            self.id2entry[idx] = entry

            w = [awx.get_width_height(self, s)[0] for s in entry]
            column_widths = map(max, zip(w, column_widths))

        for index, col in enumerate(columns):
            self.list.SetColumnWidth(index, column_widths[index])

        # Now that the list exists we can init the other base class, see wx/lib/mixins/listctrl.py
        self.itemDataMap = self.id2entry
        listmix.ColumnSorterMixin.__init__(self, len(columns))
        self.Bind(wx.EVT_LIST_COL_CLICK, self.OnColClick, self.list)

        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(text, 0, wx.EXPAND, 5)
        sizer.Add(self.list, 1, wx.EXPAND, 5)
        self.SetSizerAndFit(sizer)

        # Connect the events whose callback will be set by the client code.
        self.list.Bind(wx.EVT_LIST_ITEM_RIGHT_CLICK, self.OnRightClick)
        self.list.Bind(wx.EVT_LIST_ITEM_ACTIVATED, self.OnItemActivated)

    def makePopupMenu(self):
        """
        Creates a popup menu containing the list of
        supported actions for local file objects.
        """
        # Menu of the base class
        menu = super(VarsPanel, self).makePopupMenu()

        self.idPOPUP_VAR_SHOW = wx.NewId()
        menu.Append(self.idPOPUP_VAR_SHOW, "Show")

        # Extend base class.
        #menu.AppendSeparator()

        self.idPOPUP_VAR_EXPORT = wx.NewId()
        menu.Append(self.idPOPUP_VAR_EXPORT, "Export")

        # Extra Popup IDs for arrays
        #self.idPOPUP_ARR_SLICE = wx.NewId()
        #menu.Append(self.idPOPUP_ARR_SLICE, "Slice array")
        #self.Bind(wx.EVT_MENU, self.OnArraySlice, id=self.idPOPUP_ARR_SLICE)

        # Associate menu/toolbar items with their handlers.
        menu_handlers = [
            (self.idPOPUP_VAR_SHOW, self.onVarShow),
            (self.idPOPUP_VAR_EXPORT, self.onVarExport),
        ]

        for combo in menu_handlers:
            mid, handler = combo[:2]
            self.Bind(wx.EVT_MENU, handler, id=mid)

        return menu

    #def GetNameVarFromEvent(self, event):
    #    currentItem = event.m_itemIndex
    #    entry = self.id2entry[self.list.GetItemData(currentItem)]
    #    name = entry[0]
    #    return name, self.dataset.variables[name]

    def OnGetInfo(self, event):
        """Show extra information on the selected variables."""
        selected = getSelected(self.list)

        lines = []
        for item in selected:
            name = getColumnText(self.list, item, 0)
            var = self.dataset.variables[name]
            lines.append(str(var))
            #lines.append(ncvar_info(var))

        wxdg.ScrolledMessageDialog(self, "\n".join(lines), caption="Variable Metadata", style=wx.MAXIMIZE_BOX).Show()

    def onVarShow(self, event):
        item = getSelected(self.list)[0]
        name = getColumnText(self.list, item, 0)
        var = self.dataset.variables[name]
        text = str(var[:])
        SimpleTextViewer(self, text=text).Show()

    def onVarExport(self, event):
        """Export the selected variable to txt format and save it to file."""
        item = getSelected(self.list)[0]
        name = getColumnText(self.list, item, 0)
        var = self.dataset.variables[name]

        dialog = wx.FileDialog(self, message="Save text file", defaultDir="", defaultFile="",
                               wildcard="", style=wx.FD_SAVE | wx.FD_OVERWRITE_PROMPT)

        if dialog.ShowModal() == wx.ID_CANCEL: return

        # save the current contents in the file
        path = dialog.GetPath()

        # Save self to a text file. See :func:`np.savetext` for the description of the variables
        np.savetxt(path, var[:], fmt='%.18e')

    def OnCompare(self, event):
        """Trigger CompareEvent."""
        item = getSelected(self.list)[0]
        name = getColumnText(self.list, item, 0)
        var = self.dataset.variables[name]

        # Create and post the event to trigger the comparison.
        event = CompareEvent(id=-1, name=name, obj=var)
        wx.PostEvent(self.viewer_frame, event)

    def OnItemActivated(self, event):
        """Use `wxmplot` to plot the selected variables."""
        selected = getSelected(self.list)
        for item in selected:
            name = getColumnText(self.list, item, 0)
            var = self.dataset.variables[name]
            self.plot_variable(name, var, self.dataset)

    def plot_variable(self, var_name, var, dataset):
        """
        Use `wxmplot` to plot the selected variables.

        Args:
            var_name:
                Name of the variable
            var:
                Netcdf4 `Variable`.
            dataset:
                Netcdf4 `Dataset`.
        """
        # Remove fake dimensions.
        shape, dimensions = [], []
        for num, name in zip(var.shape, var.dimensions):
            if num > 1:
                shape.append(num)
                dimensions.append(name)

        # Get data to plot.
        data = np.reshape(var[:], shape)
        opts = self.GetPlotOptions()

        cplx_mode = opts.cplx_mode
        if cplx_mode != "None":
            if shape[-1] != 2:
                err_msg = "cplx_mode: %s. Expecting 2 as last dimensions but got %d" % (
                    cplx_mode, shape[-1])
                raise ValueError(err_msg)
            # Convert to complex then change shape and dimensions
            data = data[..., 0] + 1j*data[..., 1]
            shape = shape[:-1]
            dimensions = dimensions[:-1]
            if cplx_mode == "Abs":
                data = np.abs(data)
            elif cplx_mode == "Real":
                data = data.real
            elif cplx_mode == "Imag":
                data = data.imag
            else:
                raise ValueError("Wrong value for cplx_mode %s" % cplx_mode)

        # Plotting a scalar?
        if not shape: return
        ndim = len(shape)

        if ndim == 1:
            # Vector
            dim_name = dimensions[0]
            xx = range(len(dataset.dimensions[dim_name]))

            from wxmplot import PlotFrame
            frame = PlotFrame(parent=self)
            if opts.plot_mode == "line":
                frame.plot(xx, data)
            else:
                frame.scatterplot(xx, data)

            frame.set_xlabel(dim_name)
            frame.set_ylabel(var_name)

            frame.Show()

        elif ndim == 2:
            # Two dimensional array.
            dim_namex, dim_namey = dimensions
            xx, yy = range(len(dataset.dimensions[dim_namex])), range(len(dataset.dimensions[dim_namey]))

            mode = opts.image_mode

            if False:
                # 3d plot
                import matplotlib.pyplot as plt
                from mpl_toolkits.mplot3d import Axes3D
                fig = plt.figure()
                ax = Axes3D(fig)
                X, Y = np.meshgrid(xx, yy, sparse=False, indexing='ij')
                print(X.shape, Y.shape, data.shape)
                ax.plot_surface(X, Y, data) # rstride=8, cstride=8, alpha=0.3)
                plt.show()

            from wxmplot import ImageFrame
            frame = ImageFrame(parent=self)
            frame.display(data, title=var_name, style=mode, x=xx, y=yy, xlabel=dim_namex, ylabel=dim_namey)
            frame.Show()

        else:
            raise NotImplementedError()

    #def OnArraySlice(self, event):
    #    print("Onslice")
    #    item = getSelected(self.list)[0]
    #    name = getColumnText(self.list, item, 0)
    #    var = self.dataset.variables[name]
    #    ArraySlicerFrame(self, name, var).Show()


class ArrayComparisonFrame(wx.Frame):
    """This frame allows the user to specify how to handle array comparison."""

    def __init__(self, parent, name, data, **kwargs):
        super(ArrayComparisonFrame, self).__init__(parent, -1, **kwargs)
        self.name, self.data = name, data
        self.panel = ArrayComparisonPanel(self)

        main_sizer = wx.BoxSizer(wx.VERTICAL)

        hsizer = wx.BoxSizer(wx.HORIZONTAL)

        plot_button = wx.Button(self, -1, "Plot")
        hsizer.Add(plot_button, 0, wx.ALL | wx.ALIGN_CENTER_VERTICAL, 5)
        plot_button.Bind(wx.EVT_BUTTON, self.onPlotData)

        main_sizer.Add(hsizer, 0, wx.ALIGN_CENTER_HORIZONTAL, 5)
        main_sizer.Add(self.panel, 1, wx.EXPAND, 5)

        self.SetSizerAndFit(main_sizer)

    def onPlotData(self, event):
        operators = self.panel.GetSelectedOperators()

        from wxmplot import PlotFrame
        frame = PlotFrame(parent=self)

        xx = range(len(self.data))

        for oname, op in operators.items():
            #print(oname)
            values = [op(arr) for arr in self.data]

            frame.oplot(xx, values, label=oname)
            #if opts.plot_mode == "line":
            #    frame.oplot(xx, values)
            #else:
            #    frame.scatterplot(xx, values)

        frame.set_xlabel("File index")
        frame.set_ylabel(self.name)

        frame.Show()


class ArrayComparisonPanel(wx.Panel):
    """A panel with checkboxes used to select the scalar quantities to compare."""
    def __init__(self, parent, **kwargs):
        """
        Args:
            parent:
                Parent window.
        """
        super(ArrayComparisonPanel, self).__init__(parent, -1, **kwargs)

        main_sizer = wx.BoxSizer(wx.VERTICAL)

        static_sizer = wx.StaticBoxSizer(wx.StaticBox(self, -1, "Array contractions"), wx.VERTICAL)

        operators = [
            ("max", np.max),
            ("min", np.min),
            ("mean", np.mean),
            ("std", np.std),
            #("median", np.median),
        ]

        self.operators =  collections.OrderedDict(operators)
        self.check_boxes = collections.OrderedDict()

        for oname, _ in self.operators.items():
            cbox = wx.CheckBox(self, -1, oname, wx.DefaultPosition, wx.DefaultSize, 0)
            cbox.SetValue(True)
            static_sizer.Add(cbox, 0, wx.ALL | wx.EXPAND, 5)
            self.check_boxes[oname] = cbox

        main_sizer.Add(static_sizer, 1, wx.EXPAND, 5)

        # Add buttons to (select|deselect) all checkboxes.
        hsizer = wx.BoxSizer(wx.HORIZONTAL)

        all_button = wx.Button(self, -1, "Select all")
        all_button.Bind(wx.EVT_BUTTON, self.OnSelectAll)
        hsizer.Add(all_button, 0, wx.ALL, 5)

        deselect_button = wx.Button(self, -1, "Deselect all")
        deselect_button.Bind(wx.EVT_BUTTON, self.OnDeselectAll)
        hsizer.Add(deselect_button, 0, wx.ALL, 5)

        main_sizer.Add(hsizer, 0, wx.ALIGN_CENTER_HORIZONTAL, 5)

        self.SetSizerAndFit(main_sizer)

    def OnSelectAll(self, event):
        """Select all the check boxes."""
        for cbox in self.check_boxes.values():
            cbox.SetValue(True)

    def OnDeselectAll(self, event):
        """Deselect all the check boxes."""
        for cbox in self.check_boxes.values():
            cbox.SetValue(False)

    def GetSelectedOperators(self):
        """
        Return the list of operators selected by the user.
        """
        ops = collections.OrderedDict()

        for oname, cbox in self.check_boxes.items():
            if cbox.GetValue():
                ops[oname] = self.operators[oname]

        return ops


#class ArraySlicerFrame(wx.Frame):
#
#    def __init__(self, parent, name, var, **kwargs):
#        super(ArraySlicerFrame, self).__init__(parent, -1, **kwargs)
#        self.name, self.var = name, var
#
#        self.panel_1d = Slice1dPanel(self, name, var)
#        #self.panel_2d = Slice1dPanel(self, name, arr)
#        #self.panel_3d = Slice1dPanel(self, name, arr)
#
#        main_sizer = wx.BoxSizer(wx.VERTICAL)
#        main_sizer.Add(self.panel_1d, 1, wx.EXPAND, 5)
#        self.SetSizer(main_sizer)
#
#
#class Slice1dPanel(wx.Panel):
#    """A panel plotting for 1D slices."""
#    def __init__(self, parent, name, var, **kwargs):
#        """
#        Args:
#            parent:
#                Parent window.
#        """
#        super(Slice1dPanel, self).__init__(parent, -1, **kwargs)
#        self.name, self.var = name, var
#
#        label = wx.StaticText(self, -1, "Axis:")
#        label.Wrap(-1)
#        self.axis_choice = wx.ComboBox(self, -1, choices=var.dimensions)
#
#        hsz1 = wx.BoxSizer(wx.HORIZONTAL)
#        hsz1.Add(label, 0, wx.ALL, 5)
#        hsz1.Add(self.axis_choice, 1, wx.ALL, 5)
#
#        plot_button = wx.Button(self, -1, "Plot")
#        plot_button.Bind(wx.EVT_BUTTON, self.onPlotButton)
#        #hsizer = wx.BoxSizer(wx.HORIZONTAL)
#        #hsizer.Add(plot_button, 0, wx.ALL | wx.ALIGN_CENTER_VERTICAL, 5)
#
#        main_sizer = wx.BoxSizer(wx.VERTICAL)
#        main_sizer.Add(hsz1, 0.4, wx.ALIGN_CENTER_HORIZONTAL | wx.EXPAND, 5)
#        main_sizer.Add(plot_button, 0, wx.ALIGN_CENTER_HORIZONTAL, 5)
#        self.SetSizerAndFit(main_sizer)
#
#    def onPlotButton(self, event):
#        var = self.var
#        axis = var.dimensions.index(self.axis_choice.GetValue())
#        indices = var.shape
#        #frame = PlotFrame(parent=self)
#        #xx = range(len(self.var))
#        #frame.oplot(xx, values, label=oname)
#        #frame.set_xlabel("File index")
#        #frame.set_ylabel(self.name)
#        #frame.Show()


def wxapp_ncview(filepaths=()):
    """Standalone application."""
    app = wx.App()
    frame = NcViewerFrame(None, filepaths=filepaths)
    app.SetTopWindow(frame)
    frame.Show()

    return app



#!/usr/bin/env python
"""
Script to start the panel-based AbiPy web GUI.
"""
import os
import click
import panel as pn

from pprint import pformat
from abipy.core.release import version


@click.command()
@click.option('--port', default=5006, help="Port to listen on. Default: 5006")
@click.option('--address', default=None, help="The address the server should listen on for HTTP requests.")
@click.option('--show', default=True, help="Open server app(s) in the web browser")
@click.option('--num_procs', default=1, help="Number of worker processes for the app. Defaults to 1")
@click.option('--panel-template', "-pnt", default="FastList",
              help="Specify template for panel dasboard." +
                   "Possible values are: FastList, FastGrid, Golden, Bootstrap, Material, React, Vanilla." +
                   "Default: FastList")
@click.option('--has-remote-server', default=False, is_flag=True,
              help="True if we are running on the ABINIT server. This flag activates limitations on what the user can do." +
                   "Default: False")
@click.option("-v", '--verbose', default=0, count=True, help="Verbosity level")
@click.version_option(version=version, message='%(version)s')
def gui_app(port, address, show, num_procs, panel_template, has_remote_server, verbose):

    from abipy.panels.core import abipanel, get_abinit_template_cls_kwds, AbipyParameterized
    import abipy.panels as mod
    assets_path = os.path.join(os.path.dirname(mod.__file__), "assets")

    # Load abipy/panel extensions and set the default template
    #tmpl_kwds.update(dict(
    #    sidebar_width=240,
    #    #sidebar_width=280,
    #    #background_color="yellow",
    #))
    abipanel(panel_template=panel_template)

    print("has_remote_server:", has_remote_server)
    if has_remote_server:
        print("Enforcing limitations on what the user can do on the abinit server")
        # TODO finalize, remove files created by user
        AbipyParameterized.has_remote_server = has_remote_server

    # Import the apps and define routes for each page.
    from abipy.panels.structure import InputFileGenerator
    from abipy.panels.ddb import (PanelWithFileInput, PanelWithStructureInput, DdbPanelWithFileInput, CompareDdbWithMP,
                                  RobotWithFileInput)
    from abipy.panels.electrons import SkwPanelWithFileInput, CompareEbandsWithMP
    from abipy.panels.outputs import AbinitOutputFilePanelWithFileInput as abo_cls

    intro = """
![AbiPy Logo](assets/img/abipy_logo.png)

# AbiPy Web App

This web application exposes some of the capabilities of the [AbiPy package](https://github.com/abinit/abipy).
It consists of **multiple pages** each of which provides **specialized tools** to operate on a particular ABINIT file.
To access one of these tools, click one of the links in the sidebar or, alternatively, use the links below.
To **open/close** the sidebar, click on the Hamburger Menu Icon ☰ in the header.

Note that the **file extension** matters as the GUI won't work properly if you upload files
with extensions that are not recognized by AbiPy.

"""
    tmpl_cls, tmpl_kwds = get_abinit_template_cls_kwds()

    if verbose:
        print("Using default template:", tmpl_cls, "with kwds:\n", pformat(tmpl_kwds), "\n")

    # url --> (cls, title)
    app_routes_titles = {
        "/": (tmpl_cls, "AbiPy GUI Home"),
        "/input_generator": (InputFileGenerator, "Abinit Input Generator"),
        #"/structure_analyzer": (PanelWithFileInput(use_structure=True).get_panel(), "Structure Analyzer"),
        "/structure_analyzer": (PanelWithStructureInput, "Structure Analyzer"),
        "/outfile": (PanelWithFileInput, "Output File Analyzer"),
        "/ddb": (DdbPanelWithFileInput, "DDB File Analyzer"),
        "/abo": (abo_cls, "Abo File Analyzer"),
        "/ebands_vs_mp": (CompareEbandsWithMP, "Compare Ebands with MP"),
        "/ddb_vs_mp": (CompareDdbWithMP, "Compare DDB with MP"),
        "/skw": (SkwPanelWithFileInput, "SKW Analyzer"),
        "/robot": (RobotWithFileInput, "Robot Analyzer"),
        #"/abilog": (PanelWithFileInput().get_panel(), "DDB File Analyzer"),
        #"/state": (pn.state, "State"),
    }

    app_routes = {k: v[0] for (k, v) in app_routes_titles.items()}
    app_title = {k: v[1] for (k, v) in app_routes_titles.items()}

    for url, (cls, title) in app_routes_titles.items():
        if url in ("/", "/state"): continue
        intro += f"""

### [{title}]({url})

{cls.info_str}
"""

    main_home = pn.Column(pn.pane.Markdown(intro, sizing_mode="stretch_both"),
                          sizing_mode="stretch_both")

    # Add links to sidebar of each app so that we can navigate easily.
    links = "\n".join(f"- [{title}]({url})" for (url, title) in app_title.items())
    links = pn.Column(pn.pane.Markdown(links))

    class AppBuilder():

        def __init__(self, app_cls, sidebar_links, app_kwargs=None):
            self.app_cls = app_cls
            self.sidebar_links = sidebar_links
            self.app_kwargs = app_kwargs if app_kwargs else {}

        def build(self):
            app = self.app_cls(**self.app_kwargs)
            if hasattr(app, "get_panel"):
                app = app.get_panel()

            if hasattr(app, "sidebar"):
                app.sidebar.append(self.sidebar_links)
                #app.header.append(self.sidebar_links)

            return app

    for url, app_cls in app_routes.items():
        if url == "/":
            app_kwargs = {"main": main_home}
            app_kwargs.update(tmpl_kwds)
            app_routes[url] = AppBuilder(app_cls, sidebar_links=links, app_kwargs=app_kwargs).build
        elif url == "/state":
            app_routes[url] = app_cls
        else:
            app_routes[url] = AppBuilder(app_cls, sidebar_links=links).build

    # Call pn.serve to serve the multipage app.
    serve_kwargs = dict(
        address=address,
        port=port,
        #dev=True,
        #start=True,
        show=show,
        debug=verbose > 0,
        #title=app_title,
        num_procs=num_procs,
        static_dirs={"/assets": assets_path},
        #websocket_origin="*",
    )

    if verbose:
        print("Calling pn.serve with serve_kwargs:\n", pformat(serve_kwargs), "\n")

    pn.serve(app_routes, **serve_kwargs)


if __name__ == "__main__":
    gui_app()



#!/usr/bin/env python
"""
Script to download and install pseudopotential tables from the web
"""
from __future__ import annotations

import sys
#import os
import argparse

from abipy.core.release import __version__
from abipy.flowtk.psrepos import (tabulate_repos, repos_from_names,
                                  get_all_registered_repos, get_installed_repos_and_root)


def user_wants_to_abort():
    """Interactive problem, return False if user entered `n` or `no`."""
    try:
        answer = input("\nDo you want to continue [Y/n]")
    except EOFError:
        return False

    return answer.lower().strip() in ["n", "no"]


def abips_list(options):
    """
    List all installed pseudopotential repos.
    """
    repos, repos_root = get_installed_repos_and_root()

    if not repos:
        print("Could not find any pseudopotential repository installed in:", repos_root)
        return 0

    print(f"The following pseudopotential repositories are installed in {repos_root}:\n")
    print(tabulate_repos(repos, exclude=["installed"], verbose=options.verbose))

    #if options.verbose:
    #    for repo in repos:
    #        if repo.ispaw: continue
    #        pseudos = repo.get_pseudos(table_name="standard")
    #        print(pseudos)
    #else:
    #    print("\nUse -v to print the pseudos")

    if not options.checksums:
        print("\nUse -c to validate the md5 checksum")
        return 0

    exc_list = []
    for repo in repos:
        try:
            repo.validate_checksums(options.verbose)
        except Exception as exc:
            exc_list.append(exc)

    if exc_list:
        print("\nList of exceptions raised by validate_checksums:")
        for exc in exc_list:
            print(exc)

    return len(exc_list)


def abips_avail(options):
    """
    Show available repos.
    """
    print("List of available pseudopotential repositories:\n")
    all_repos = get_all_registered_repos()
    print(tabulate_repos(all_repos, with_citations=True, verbose=options.verbose))


#def abips_nc_install(options):
#    """
#    Get NC repo. Can choose among three formats: psp8, upf2 and psml.
#    By default we fetch all formats.
#    """
#    repos_root = get_repos_root(options)
#    all_repos = get_all_registered_repos()
#    repos = [repo for repo in ALL_REPOS if repo.isnc and not repo.is_installed(repos_root)]
#    if not repos:
#        print(f"All registered NC repositories are already installed in {repos_root}. Returning")
#        return 0
#
#    print("The following NC repositories will be installed:\n")
#    pprint_repos(repos, repos_root=repos_root)
#    if not options.yes and user_wants_to_abort():
#        return 2
#
#    print("Fetching NC repositories. It may take some time ...")
#    for repo in repos:
#        repo.install(repos_root, options.verbose)
#
#    if options.verbose: abips_list(options)
#    return 0
#
#
#def abips_paw_install(options):
#    """
#    Get PAW repositories in PAWXML format.
#    """
#    repos_root = get_repos_root(options)
#    all_repos = get_all_registered_repos()
#    repos = [repo for repo in ALL_REPOS if repo.ispaw and not repo.is_installed(repos_root)]
#    if not repos:
#        print(f"All registered PAW repositories are already installed in {repos_root}. Returning")
#        return 0
#
#    print("The following PAW repositories will be installed:")
#    pprint_repos(repos, repos_root=repos_root)
#    if not options.yes and user_wants_to_abort():
#        return 2
#
#    print("Fetching PAW repositories. It may take some time ...")
#    for repo in repos:
#        repo.install(repos_root, options.verbose)
#
#    if options.verbose: abips_list(options)
#    return 0


def abips_install(options):
    """
    Install pseudopotential repositories by name(s).
    Use the `avail` command to get the repo name.
    """
    repos = repos_from_names(options.repo_names)
    repos = [repo for repo in repos if not repo.is_installed()]

    if not repos:
        print("Table(s) are already installed! Nothing to do. Returning")
        return 0

    print("The following pseudopotential repositories will be installed:")
    print(tabulate_repos(repos, verbose=options.verbose))
    #if not options.yes and user_wants_to_abort(): return 2

    for repo in repos:
        repo.install(verbose=options.verbose)

    if options.verbose: abips_list(options)
    return 0


def abips_show(options):
    """
    Show pseudopotential tables
    """
    repos = repos_from_names(options.repo_names)
    repos = [repo for repo in repos if repo.is_installed()]

    if not repos:
        print(f"There's no installed repository with name in: {options.repo_names}")
        return 1

    for repo in repos:
        print(repo)
        table_names = ["standard", "stringent"]
        for table_name in table_names:
            pseudos = repo.get_pseudos(table_name=table_name)
            #for pseudo in pseudos:
            #    print(pseudo.filepath)
            #    print(pseudo.as_dict()["filepath"])
            print(f"For accuracy: {table_name}:")
            #print(pseudos.summarize())
            print(pseudos)

    return 0


def get_epilog():
    return """\

Usage example:

  abips.py avail                             --> Show all registered pseudopotential repositories
  abips.py list                              --> List repositories installed on this machine
  abips.py install ONCVPSP-PBEsol-SR-PDv0.4  --> Install repository by name (requires internet connection).
"""

  #abips.py show ONCVPSP-PBEsol-SR-PDv0.4  --> Install repository.
  #abips.py nc_install                        --> Get all NC repositories (most recent version)
  #abips.py nc_install -xc PBE -fr -sr --version 0.4
  #abips.py paw_install                        --> Get all PAW repositories (most recent version)


def get_parser(with_epilog=False):

    # Parent parser for common options.
    copts_parser = argparse.ArgumentParser(add_help=False)
    copts_parser.add_argument('-v', '--verbose', default=0, action='count', # -vv --> verbose=2
                              help='verbose, can be supplied multiple times to increase verbosity.')

    copts_parser.add_argument('--loglevel', default="ERROR", type=str,
                              help="Set the loglevel. Possible values: CRITICAL, ERROR (default), WARNING, INFO, DEBUG")

    #copts_parser.add_argument('--repos-root', "-r", type=str,
    #                          default=os.path.expanduser(os.path.join("~", ".abinit", "pseudos")),
    #                          help='Installation directory. Default: $HOME/.abinit/pseudos')

    #copts_parser.add_argument('-y', "--yes", action="store_true", default=False,
    #                          help="Do not ask for confirmation when installing repositories.")

    # Build the main parser.
    parser = argparse.ArgumentParser(epilog=get_epilog() if with_epilog else "",
                                     formatter_class=argparse.RawDescriptionHelpFormatter)
    parser.add_argument('-V', '--version', action='version', version=__version__)

    # Create the parsers for the sub-commands
    subparsers = parser.add_subparsers(dest='command', help='sub-command help', description="Valid subcommands")

    # Subparser for avail command.
    subparsers.add_parser("avail", parents=[copts_parser], help=abips_avail.__doc__)

    # Subparser for list command.
    p_list = subparsers.add_parser("list", parents=[copts_parser], help=abips_list.__doc__)
    p_list.add_argument("-c", "--checksums", action="store_true", default=False,
                        help="Validate md5 checksums")

    # Subparser for install command.
    p_install = subparsers.add_parser("install", parents=[copts_parser], help=abips_install.__doc__)
    p_install.add_argument("repo_names", type=str, nargs="+", help="List of repositories to download.")
    p_install.add_argument("-c", "--checksums", action="store_true", default=False,
                           help="Validate md5 checksums")

    # Subparser for nc_install command.
    #p_nc_install = subparsers.add_parser("nc_install", parents=[copts_parser], help=abips_nc_install.__doc__)

    # Subparser for paw_install command.
    #p_paw_install = subparsers.add_parser("paw_install", parents=[copts_parser], help=abips_paw_install.__doc__)

    # Subparser for show command.
    p_show = subparsers.add_parser("show", parents=[copts_parser], help=abips_show.__doc__)
    p_show.add_argument("repo_names", type=str, nargs="+", help="List of Repo names.")

    return parser


def main():

    def show_examples_and_exit(err_msg=None, error_code=1):
        """Display the usage of the script."""
        sys.stderr.write(get_epilog())
        if err_msg: sys.stderr.write("Fatal Error\n" + err_msg + "\n")
        sys.exit(error_code)

    parser = get_parser(with_epilog=True)

    # Parse command line.
    try:
        options = parser.parse_args()
    except Exception as exc:
        print(exc)
        show_examples_and_exit(error_code=1)

    # loglevel is bound to the string value obtained from the command line argument.
    # Convert to upper case to allow the user to specify --loglevel=DEBUG or --loglevel=debug
    import logging
    numeric_level = getattr(logging, options.loglevel.upper(), None)
    if not isinstance(numeric_level, int):
        raise ValueError('Invalid log level: %s' % options.loglevel)
    logging.basicConfig(level=numeric_level)

    #if options.repos_root:

    return globals()[f"abips_{options.command}"](options)


if __name__ == "__main__":
    sys.exit(main())
